#!/usr/bin/env python3

"""
Author:         xiazeng
Create time:    2020/8/28
Description:    

"""
import logging
from attestbase import AtTestBase


class AdbTest(AtTestBase):
    def test_file_exists(self):
        ret = self.at.adb.file_exists("/sdcard/Android")
        self.assertTrue(ret)
        ret = self.at.adb.file_exists("/sdcard/Androidd")
        self.assertFalse(ret)

    def test_get_act(self):
        act = self.at.adb.get_current_activity()
        print(act)


#!/usr/bin/env python3

"""
Author:         xiazeng
Create time:    2021/5/14
Description:    

"""
import logging

import attestbase
import at.webdriver.driver

logging.basicConfig(level=logging.DEBUG)


class DriverTest(attestbase.AtTestBase):
    def test_click(self):
        pid = self.at.adb.get_android_pid("com.tencent.mm:tools")
        web_sock = at.webdriver.driver.get_local_sock(self.at.serial, "xweb_devtools_remote_%d" % pid)
        page = at.webdriver.driver.WebPage(web_sock, self.at.adb.serial, self.at)
        page.by_text_contains("统计").click()
        page.close()

    def test_click_lr(self):
        pid = self.at.adb.get_android_pid("com.tencent.mm:toolsmp")
        web_sock = at.webdriver.driver.get_local_sock(self.at.serial, "xweb_devtools_remote_%d" % pid)
        page = at.webdriver.driver.WebPage(web_sock, self.at.adb.serial, self.at)
        texts = page.invoke_js('getTexts')
        for text in texts:
            logging.info(text)
        web_sock.close()
        return texts



# -*- coding: utf-8 -*-
import threading
import sys

import time
import logging

__author__ = 'xiazeng'
logger = logging.getLogger()


class KThread(threading.Thread):
    """A subclass of threading.Thread, with a kill()
    method.
    Come from:
    Kill a thread in Python:
    http://mail.python.org/pipermail/python-list/2004-May/260937.html
    """
    def __init__(self, *args, **kwargs):
        threading.Thread.__init__(self, *args, **kwargs)
        self.killed = False

    def start(self):
        """Start the thread."""
        self.__run_backup = self.run
        self.run = self.__run  # Force the Thread to install our trace.
        threading.Thread.start(self)

    def __run(self):
        """Hacked run function, which installs the
        trace."""
        sys.settrace(self.globaltrace)
        self.__run_backup()
        self.run = self.__run_backup

    def globaltrace(self, frame, why, arg):
        if why == 'call':
            return self.localtrace
        else:
            return None

    def localtrace(self, frame, why, arg):
        if self.killed:
            if why == 'line':
                raise SystemExit()
        return self.localtrace

    def kill(self):
        self.killed = True


class Timeout(Exception):
    """function run timeout"""
    pass


def timeout(seconds):
    """超时装饰器，指定超时时间
    若被装饰的方法在指定的时间内未返回，则抛出Timeout异常"""

    def timeout_decorator(func):
        """真正的装饰器"""

        def _new_func(old_func, result, old_func_args, old_func_kwargs):
            result.append(old_func(*old_func_args, **old_func_kwargs))

        def _(*args, **kwargs):
            result = []
            new_kwargs = {  # create new args for _new_func, because we want to get the func return val to result list
                'old_func': func,
                'result': result,
                'old_func_args': args,
                'old_func_kwargs': kwargs
            }
            thd = KThread(target=_new_func, args=(), kwargs=new_kwargs)
            thd.start()
            thd.join(seconds)
            alive = thd.isAlive()
            thd.kill()  # kill the child thread
            if alive:
                raise Timeout(u'function run too long, timeout %d seconds.' % seconds)
            else:
                return result[0] if len(result) > 0 else None

        _.__name__ = func.__name__
        _.__doc__ = func.__doc__
        return _
    return timeout_decorator


def retry_in(max_seconds, sleep_seconds=1):
    """
    重试装饰器，在指定的时间内不断重试
    被装饰的方法如果返回非真，那么认为需要重试。如果超时都未返回真，那么以最后一次测试的结果返回
    注意： 如果函数内部发生异常，将捕捉不到堆栈
    """

    def timeout_decorator(func):
        """真正的装饰器"""

        def _new_func(old_func, result, old_func_args, old_func_kwargs, ex):
            try:
                ret = old_func(*old_func_args, **old_func_kwargs)
                result.append(ret)
            except Exception as e:
                ex.append(e)

        def _(*args, **kwargs):
            ex = []
            result = []
            new_kwargs = {  # create new args for _new_func, because we want to get the func return val to result list
                'old_func': func,
                'result': result,
                'old_func_args': args,
                'old_func_kwargs': kwargs,
                'ex': ex
            }
            index = 0
            last_time = time.time() + max_seconds
            while time.time() < last_time:
                thd = KThread(target=_new_func, args=(), kwargs=new_kwargs)
                thd.start()
                thd.join(max_seconds)
                thd.kill()  # kill the child thread
                if len(ex) == 1:
                    raise ex[0]
                if index < len(result) and result[index]:
                    break
                index += 1
                time.sleep(sleep_seconds)
            else:
                return result[-1] if result else None
            return result[index]

        _.__name__ = func.__name__
        _.__doc__ = func.__doc__
        return _
    return timeout_decorator


def retry_exception(ExceptionToCheck=Exception, tries=3, delay=1, backoff=2):
    """Retry calling the decorated function using an exponential backoff.

    http://www.saltycrane.com/blog/2009/11/trying-out-retry-decorator-python/
    original from: http://wiki.python.org/moin/PythonDecoratorLibrary#Retry

    :param ExceptionToCheck: the exception to check. may be a tuple of
        exceptions to check
    :type ExceptionToCheck: Exception or tuple
    :param tries: number of times to try (not retry) before giving up
    :type tries: int
    :param delay: initial delay between retries in seconds
    :type delay: int
    :param backoff: backoff multiplier e.g. value of 2 will double the delay
        each retry
    :type backoff: int
    :param logger: logger to use. If None, print
    :type logger: logging.Logger instance
    """
    def deco_retry(f):

        def f_retry(*args, **kwargs):
            mtries, mdelay = tries, delay
            while mtries > 1:
                try:
                    return f(*args, **kwargs)
                except ExceptionToCheck as e:
                    msg = u"%s, Retrying in %d seconds..." % (e, mdelay)
                    logger.exception(msg)
                    time.sleep(mdelay)
                    mtries -= 1
                    mdelay *= backoff
            return f(*args, **kwargs)

        return f_retry  # true decorator

    return deco_retry

method_costs = {}


def cost_log(func):
    def wrap(*arg, **kargs):
        start = time.time()
        r = func(*arg, **kargs)
        end = time.time()
        if func not in method_costs:
            method_costs[func] = {"total": 0, "num": 0}
        method_costs[func]["total"] += (end - start)*1000
        method_costs[func]["num"] += 1
        logger.info(" %s  cost %d ms" % (func.__name__, (end - start)*1000))
        return r
    return wrap


class cached_property(object):
    """ A property that is only computed once per instance and then replaces
        itself with an ordinary attribute. Deleting the attribute resets the
        property.

        Source: https://github.com/bottlepy/bottle/commit/fa7733e075da0d790d809aa3d2f53071897e6f76
        """

    def __init__(self, func):
        self.__doc__ = getattr(func, '__doc__')
        self.func = func

    def __get__(self, obj, cls):
        if obj is None:
            return self
        value = obj.__dict__[self.func.__name__] = self.func(obj)
        return value


def get_key(i):
    if isinstance(i, str):
        return hash(i)
    if isinstance(i, int):
        return i
    if isinstance(i, object):
        return hash(i)
    raise RuntimeError("notice xia zeng")


def get_id_tuple(f, args, kwargs, mark=object()):
    l = [id(f)]
    for arg in args:
        l.append(get_key(arg))
    l.append(get_key(mark))
    for k, v in kwargs:
        l.append(k)
        l.append(get_key(v))
    return tuple(l)

class memorize(dict):
    def __init__(self, func):
        self.func = func

    def __call__(self, *args):
        return self[args]

    def __missing__(self, key):
        result = self[key] = self.func(*key)
        return result


class testCachedProperty(object):

    @cached_property
    def very_slow(self):
        """Represents a performance heavy property."""
        time.sleep(1)  # Wait a WHOLE second!
        return "I am slooooow"


@timeout(5)
def test_timeout(seconds, text):
    print('start', seconds, text)
    time.sleep(seconds)
    print('finish', seconds, text)
    return seconds


@retry_in(5, 1)
def test_retry_in(seconds, text):
    print('start', seconds, text)
    if seconds > 5:
        return True


def test():
    a = testCachedProperty()
    for sec in range(1, 10):

        try:
            print('*' * 20)
            # print test_timeout(sec, 'test waiting %d seconds' % sec)
            # print test_retry_in(sec, 'test waiting %d seconds' % sec)

            print(a.very_slow)
        except Timeout as e:
            print(e)


if __name__ == '__main__':
    test()



# -*- coding: utf-8 -*-
"""
@author: 'xiazeng'
@created: 2017/1/17 
"""
import base64
import json
import logging
import time

from at.core import element, javadriver, uixml, case_repair_adapter
import at.utils.commonhelper as commonhelper

logger = logging.getLogger()


class PyUiDevice(object):
    def __init__(self, jd):
        self.jd = jd  # type: javadriver.JavaDriver
        self.adb = jd.adb
        self.input_interceptor = None
        self._width = None
        self._height = None

    def set_input_interceptor(self, input_interceptor):
        self.input_interceptor = input_interceptor

    def enter(self, text):
        self.record(u"直接输入: %s", text)
        self.request_method("setText", [text])

    def enter_zh(self, zh_text):
        self.record(u"直接输入中文: %s", zh_text)
        self.request_method("setChineseText", [zh_text])

    def get_ui_view(self, selector, instance=0, views=None):
        """
        :rtype: uixml.UiView
        """
        if views is None:
            views = self.jd.get_ui_views()
            if isinstance(selector, element.Element):
                for view in views:
                    if selector.match_ui_view(view):
                        return view
            else:
                logger.error("unknown type: %s, %s", type(selector), selector)

        return uixml.get_view(views, selector, instance)

    def click_on_ui_view(self, ui_view):
        """
        :type ui_view: uixml.UiView
        :return: 
        """
        self.click_on_point(ui_view.center_x, ui_view.center_y)

    def gesture(self, points):
        self.record(u"手势: %s", json.dumps(points))
        params = []
        for point in points:
            params.append(int(point[0]))
            params.append(int(point[1]))
        self.jd.request_at_device("gesture", params)

    @case_repair_adapter.deco_do_action
    def click_on_point(self, x, y):
        self.record(u"点击坐标: %s, %s", int(x), int(y))
        if self.input_interceptor:
            return self.input_interceptor.click(x, y)
        else:
            self.request_method("click", [int(x), int(y)])

    def click_on_list(self, xy_list, max_sleep=0.5, min_sleep=0.5):
        """
        点击列表
        :param xy_list:
        :param interval: 两次点击直接最大停顿时间
        :param min_sleep: 两次点击直接最小停顿时间
        :return:
        """
        self.jd.push_op(u"点击列表点:%s", xy_list)
        for x, y in xy_list:
            t = time.time()
            self.request_method("click", [int(x), int(y)])
            sleep_time = max_sleep - (time.time() - t)
            if sleep_time > min_sleep:
                time.sleep(sleep_time)
            else:
                time.sleep(min_sleep)

    def screen_point_data(self, x, y, width, height, quality):
        self.record(u"局部截图: %s, %s, %s, %s", int(x), int(y), int(width), int(height))
        base64_str = self.request_method("screenPoint", [int(x), int(y), int(width), int(height), int(quality)])
        # logger.info("base64_str is %s"%base64_str)
        base64_data = base64.b64decode(base64_str)
        return base64_data

    def screen_point(self, filename, x, y, width, height, quality=30):
        d = self.screen_point_data(x, y, width, height, quality)
        open(filename, "wb").write(d)

    def click_on_center(self):
        self.click_on_point(self.width()/2, self.height()/2)

    def swipe(self, x1, y1, x2, y2, steps):
        self.record(u"滑动: %s,%s -> %s, %s", x1, y1, x2, y2)
        if self.input_interceptor:
            self.input_interceptor.swipe(x1, y1, x2, y2, steps)
        else:
            self.fast_scroll(x1, y1, x2, y2, 100, steps)
            # self.request_method("swipe", [int(x1), int(y1), int(x2), int(y2), steps])

    def swipe_left(self, rect, steps=10):
        y = rect.top + rect.height / 2
        x2, x1 = rect.left + rect.width / 4, rect.left + rect.width * 3 / 4
        self.swipe(x1, y, x2, y, steps)

    def swipe_right(self, rect, steps=20):
        y = rect.top + rect.height / 2
        x1, x2 = rect.left + rect.width / 4, rect.left + rect.width * 3 / 4
        self.swipe(x1, y, x2, y, steps)

    def long_click_on_point(self, x, y, duration=3*1000):
        self.record(u"长按: %s, %s, 时间:%sms", int(x), int(y), duration)
        if self.input_interceptor:
            self.input_interceptor.long_click(x, y, duration/1000.0)
        else:
            self.request_method("longClick", [int(x), int(y), duration])

    def long_click_on_center(self, duration=3*1000):
        self.long_click_on_point(self.width()/2, self.height()/2, duration)

    def get_rotation(self):
        """
        https://developer.android.com/reference/android/view/Surface.html#ROTATION_0
        :return: 0-3
        """
        return self.request_method("getRotation")

    def status_bar_height(self):
        """获取状态栏的高度"""
        return self.request_method("getStatusBarHeight")

    def height(self):
        """
        获取全屏幕大小（包括status_bar的高度）
        """
        if not self._height:
            self._height = self.request_method("getDisplayHeight")
        return self._height

    def width(self):
        if not self._width:
            self._width = self.request_method("getDisplayWidth")
        return self._width

    def device_size(self):
        return self.width(), self.height()

    def wait_window_update(self, pkg, timeout=10*1000):
        ret = self.request_method("waitForWindowUpdate", [pkg, timeout])
        if ret:
            time.sleep(2)
        return ret

    def wait_for_idle(self, timeout=10*1000):
        """
        等待一段时间内ui没有变化
        """
        return self.request_method("waitForIdle", [timeout])

    def current_package(self):
        return self.request_method("getCurrentPackageName")

    def drag(self, x1, y1, x2, y2, steps=10, first_sleep=200, last_sleep=200):
        """
        拖动
        """
        self.record(u"拖动: %s,%s -> %s, %s", x1, y1, x2, y2)
        self.custom_drag(x1, y1, x2, y2, [first_sleep]+steps*[50]+[last_sleep])

    def scroll(self, x1, y1, x2, y2, steps=20):
        """
        滑动
        """
        self.record(u"滑动: %s,%s -> %s, %s", x1, y1, x2, y2)
        self.jd.request_action(self.jd.ACTION_AT_DEVICE, "fastDrag", [int(x1), int(y1), int(x2), int(y2), 500, steps])

    def fast_scroll_one_forth_page(self, direction='up', num=1, milliseconds=100, steps=10):
        """
        滑动1/4个屏幕
        """
        width = self.width()
        height = self.height() - self.status_bar_height()
        for i in range(num):
            if direction == "up":
                self.fast_scroll(width / 2, height * 5 / 8, width / 2, height * 3 / 8, milliseconds, steps)
            elif direction == "down":
                self.fast_scroll(width / 2, height * 3 / 8, width / 2, height * 5 / 8, milliseconds, steps)
            elif direction == "left":
                self.fast_scroll(width*5/8, height/2, width*3/8, height/2, milliseconds, steps)
            elif direction == "right":
                self.fast_scroll(width*3/8, height/2, width*5/8, height/2, milliseconds, steps)
            else:
                raise TypeError("%s direction" % direction)
            time.sleep(0.1)

    def scroll_one_forth_page(self, direction, num=1):
        """
        滑动1/4个屏幕,direction是方向：up是从下往上, down是从上往下，left是从右到左，right是从左到右
        """
        width = self.width()
        height = self.height() - self.status_bar_height()
        self.jd.set_capture_op(False)
        for i in range(num):
            if direction == "up":
                self.scroll(width/2, height*5/8, width/2, height*3/8)
            elif direction == "down":
                self.scroll(width / 2, height*3/8, width / 2, height*5/8)
            elif direction == "left":
                self.scroll(width*5/8, height/2, width*3/8, height/2)
            elif direction == "right":
                self.scroll(width*3/8, height/2, width*5/8, height/2)
            else:
                raise TypeError("%s direction" % direction)
            time.sleep(0.2)
        self.jd.set_capture_op(True)
        self.record("滑动:%s", direction)

    def fast_scroll_one_page(self, direction, num=1, milliseconds=100, steps=1):
        """
        快速滑动一整屏幕
        """
        width = self.width()
        height = self.height() - self.status_bar_height()
        for i in range(num):
            if direction == "up":
                self.fast_scroll(width/2, height*7/8, width/2, height*1/8, milliseconds, steps)
            elif direction == "down":
                self.fast_scroll(width / 2, height*1/8, width / 2, height*7/8, milliseconds, steps)
            elif direction == "left":
                self.fast_scroll(width*7/8, height/2, width*1/8, height/2, milliseconds, steps)
            elif direction == "right":
                self.fast_scroll(width*1/8, height/2, width*7/8, height/2, milliseconds, steps)
            else:
                raise TypeError("%s direction" % direction)
            time.sleep(0.1)

    def scroll_one_page(self, direction):
        return self.scroll_one_forth_page(direction, 4)

    def fast_scroll(self, x1, y1, x2, y2, milliseconds=500, steps=10):
        self.record(u"快速滑动: %s,%s -> %s, %s 时间:%s", x1, y1, x2, y2, milliseconds)
        self.jd.request_action(self.jd.ACTION_AT_DEVICE, "fastDrag", [int(x1), int(y1), int(x2), int(y2), milliseconds, steps])

    def custom_drag(self, x1, y1, x2, y2, sleep_times):
        self.jd.request_action(self.jd.ACTION_AT_DEVICE, "customDrag", [int(x1), int(y1), int(x2), int(y2), json.dumps(sleep_times)])

    def request_method(self, method, params=None):
        return self.jd.request_at_device(method, params)

    def wake_up(self):
        """
        等待一段时间内ui没有变化
        """
        return self.request_method("wakeUp")

    # 处理rect相关的操作
    def screen_rect(self, filename, rect, quality=30):
        d = self.screen_point_data(rect.left, rect.top, rect.width, rect.height, quality)
        open(filename, "wb").write(d)

    def click(self, rect):
        self.click_on_point(rect.center_x, rect.center_y)

    def long_click(self, rect, microseconds=3*1000):
        self.long_click_on_point(rect.center_x, rect.center_y, microseconds)

    def screen_shot(self, filename, scale=1.0, quality=-1, max_idle=200, display_id=None):
        if display_id is None:
            try:
                scale = scale if 0 < scale < 1.0 else 1
                quality = quality if 0 < quality < 100 else 30
                res = self.jd.request_at_device("screen", [float(scale), quality, max_idle])
                if not res:
                    logger.error("uiautomator screen failed: response empty. filename: %s. try reconnect" % filename)
                    self.jd.reconnect()
                    raise RuntimeError("uiautomator screen response empty")
                commonhelper.base64_to_file(res, filename)
                return filename
            except Exception:
                logger.exception("uiautomator screen failed")
        self.adb.screen(filename, display_id)
        return filename

    def record(self, msg, *args):
        if self.input_interceptor:
            self.jd.push_op(msg, *args, display=self.input_interceptor.display_id)
        else:
            self.jd.push_op(msg, *args)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@Author: lockerzhang
@LastEditors: lockerzhang
@Description: file content
@Date: 2019-03-11 20:34:28
@LastEditTime: 2019-03-11 20:52:04
"""

from enum import Enum


class MediaType(Enum):
    """
    媒体类型
    """

    UNKNOW = 1
    VIDEO = 2
    AUDIO = 3
    LIVE_PLAY = 4
    LIVE_PUSH = 5


class MediaStatus(Enum):
    """
    媒体状态
    """

    PLAYING = 10
    STOP = 11
    RESUME = 12
    PAUSE = 13
    PUSHING = 14


class AppStatus(Enum):
    """
    APP 活动状态
    """

    FOREGROUND = 1001
    BACKGROUND = 1002


class Orientation(Enum):
    """
    屏幕方向
    """

    ORIENTATION_UP = 1010
    ORIENTATION_DOWN = 1011
    ORIENTATION_LEFT = 1012
    ORIENTATION_RIGHT = 1013



#!/usr/local/bin/python2.7
# encoding: utf-8
import logging
from . import javadriver


logger = logging.getLogger()


class AccessHelper(object):
    def __init__(self, jd):
        self.jd = jd    # type: javadriver.JavaDriver
        self.adb = jd.adb

    def screen_when_click_sys_dialog(self, true_or_false):
        """
        在处理系统弹框的时候截图
        :param true_or_false: 
        :return: 
        """
        return self.jd.request_sys_handler("setScreenBeforeClick", [true_or_false])

    def set_app_dialog_pkg(self, pkg_name):
        self.jd.request_dialog_handler("setPkgName", [pkg_name])

    def add_rid(self, rid):
        self.jd.request_dialog_handler("addRid", [rid])

    def set_click_sys_dialog(self, true_or_false):
        return self.jd.request_sys_handler("setPermissionMonitor", [true_or_false])

    def get_sys_dialog_click_info(self):
        return self.jd.request_sys_handler("getMonitorScreen")


# encoding=utf-8


class WsPools(object):
    """
    Websocket连接池，保存所有的websocket连接
    - 单例模式
    - tab的id和url都是有可能重复出现的, 最好不要用成主键来索引, 否则可能导致多个线程的竞态条件产生
    """
    _socket_map = {}

    @classmethod
    def add(cls, sock):
        "按主键存储"
        cls._socket_map[sock.identity] = sock

    @classmethod
    def remove(cls, sock):
        "按主键移除"
        del cls._socket_map[sock.identity]

    @classmethod
    def get(cls, identity):
        "根据主键返回"
        return cls._socket_map[identity]

    @classmethod
    def has(cls, identity):
        "判断是否存在"
        return identity in WsPools._socket_map

    # =======================================================================================
    # Business ID
    # =======================================================================================

    @classmethod
    def has_id(cls, tab_id):
        "判断指定业务ID页面是否存在"
        for sock in WsPools._socket_map.values():
            if sock.tab_id == tab_id:
                return True
        return False

    @classmethod
    def get_by_id(cls, tab_id):
        "根据其业务ID返回"
        for sock in WsPools._socket_map.values():
            if sock.tab_id == tab_id:
                return sock



#!/usr/bin/env python
# encoding:utf-8
import logging

from attestbase import AtTestBase

logger = logging.getLogger()


class ElementTest(AtTestBase):
    @property
    def e(self):
        return self.at.e

    def test_action(self):
        self.e.text(u"发现").click()
        self.e.list_view().swipe_left()
        self.e.list_view().swipe_right()
        self.e.text(u"朋友圈").click()
        self.e.list_view().swipe_up()
        self.e.list_view().swipe_down()

    def test_input(self):
        self.e.edit_text().enter("hello world!")
        self.e.edit_text().enter("hello ", is_clear_text=False)
        self.e.edit_text().enter("world", is_click=False)

    def test_enter_sns(self):
        a = self.at
        a.e.text("发现").click()
        a.e.text("朋友圈").click()
        a.e.desc("拍照分享").long_click()
        a.e.text("我知道了").click_if_exists()
        a.e.edit_text().enter("发个纯文本朋友圈")
        a.e.text("发表").click()
        a.e.text("发个纯文本朋友圈").assert_exists()

    def test_mm_enter(self):
        # self.at.device.enter("测试")
        # self.at.e.edit_text().enter("测试")
        self.at.e.cls_name("android.widget.EditText").enter("测试")  # com.tencent.mm.ui.widget.cedit.edit.CustomEditText

    def test_exists(self):
        print(self.at.e.pkg("com.tencent.mm").desc(u"图片").exists())

    def test_handle_modal(self):
        self.at.e.text("")

    def test_index_click(self):
        for ui_views in self.at.java_driver.dump_all_views():
            logging.info("=" * 120)
            for ui_view in ui_views:
                logging.info(ui_view)
        self.e.rid("com.tencent.mm:id/d8a").index(6).click()
        self.e.rid("android.view.View").index()



#!/usr/local/bin/python3
# -*- coding: utf-8 -*-
"""
Author:         lockerzhang
Filename:       minilimit.py
Create time:    2019/12/23 17:49
Description:

"""
from functools import wraps

__all__ = ["limit"]


def limit(platform="all", app="all", pack_name=None, sort=None):
    """
    用例装饰器
    :param platform: 系统平台
    :param app: 应用
    :param pack_name: 打包名称
    :param sort: 包内排序
    :return:
    """

    def set_limit(func):
        setattr(func, "platform", platform)
        setattr(func, "app", app)
        setattr(func, "pack_name", pack_name)
        setattr(func, "sort", sort)

        @wraps(func)
        def wrapper(*args, **kwargs):
            rtn = func(*args, **kwargs)
            return rtn

        return wrapper

    return set_limit


@limit(platform="ios", app="wx", pack_name="pkg1")
def casetest():
    print(1111)


if __name__ == "__main__":
    module = __import__("minilimit")
    attr = dir(module)
    func = getattr(module, "casetest")
    func_attr = dir(func)
    platform = getattr(func, "platform")
    app = getattr(func, "app")
    pack_name = getattr(func, "pack_name")
    sort = getattr(func, "sort")
    print(22222)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import os.path
import time
import wda
import logging

wda.DEBUG = True  # default False
wda.HTTP_TIMEOUT = 60.0  # default 60.0 seconds

logger = logging.getLogger()
WORKSPACE_DIR = os.path.dirname(os.path.dirname(__file__))
sys.path.insert(0, WORKSPACE_DIR)


class UiAutomatorException(Exception):
    def __init__(self, message=None, reason=None):
        self._message = message
        self._reason = reason

    def print_message(self):
        logger.debug(self._message)

    def print_reason(self):
        logger.debug(self._reason)

    @property
    def message(self):
        if not self._message:
            self._message = "No Message"
        return self.__name__, "message:", self._message

    @property
    def reason(self):
        if not self._reason:
            self._reason = "No Reason"
        return self.__name__, "reason:", self._reason

    @message.setter
    def message(self, value):
        self._message = value

    @reason.setter
    def reason(self, value):
        self._reason = value


class NoSuchElementError(UiAutomatorException):
    def __init__(self, message="Cannot find this elements", reason=None):
        super(NoSuchElementError, self).__init__(message, reason)
        self._message = message
        self._reason = reason


class ObjectInstanceEqualError(UiAutomatorException):
    pass


class ArgsMissingError(UiAutomatorException):
    pass


class NoSuchObjectException(UiAutomatorException):
    pass


sleep_time = 1.5


class WdaUI(object):
    def __init__(self, server_url=None, bundle_id=None):
        if server_url is None:
            server_url = "http://localhost:8100"
        logger.info("启动 %s" % bundle_id)
        self.client = wda.Client(server_url)
        self.session = self.client.session(bundle_id)
        self._height = 0
        self._width = 0

    def set_alert_callback(self, callback):
        self.session.set_alert_callback(callback)

    def close(self):
        self.session.close()

    # 切后台若干秒再切回来
    def deactivate(self, duration):
        self.client.deactivate(duration)

    @property
    def width(self):
        self._width = self.session.window_size()[0]
        return self._width

    @property
    def height(self):
        self._height = self.session.window_size()[1]
        return self._height

    def device_size(self):
        size = self.session.window_size()
        return size

    def press_home(self):
        self.client.home()

    def status(self):
        return self.client.status()

    def source(self, data_format="xml"):
        return self.client.source(data_format)

    def screen_shot(self, filename):
        return self.client.screenshot(filename)

    def swipe(self, start_x, start_y, end_x, end_y, duration=0.1):
        self.session.drag(start_x, start_y, end_x, end_y, duration)

    def click_on_text(self, text, class_name=None, index=0):
        # type: (object, object, object) -> object
        self.session(text=text, class_name=class_name, index=index).tap()
        time.sleep(sleep_time)

    def click_on_partial_text(self, partial_text, class_name=None, index=0):
        self.session(partial_text=partial_text, class_name=class_name, index=index).tap()
        time.sleep(sleep_time)

    def click_point(self, x, y):
        self.session.click(x, y)
        # self.session.tap(x, y)

    def click_on_center(self):
        center_x = self.width / 2
        center_y = self.height / 2
        self.click_point(center_x, center_y)

    def click_on_element_id(self, element_id):
        self.session.tapElement(element_id)

    def long_click_point(self, x, y, duration=3):
        self.session.long_tap(x, y, duration=duration)

    def long_click_view(self, class_name, text=None, index=0, duration=3):
        self.session(class_name=class_name, text=text, index=index).tap_hold(duration=duration)
        time.sleep(sleep_time)

    def long_click_text(self, text, duration=3, index=0):
        self.session(text=text, index=index).tap_hold(duration=duration)
        time.sleep(sleep_time)

    def double_click_point(self, x, y):
        self.session.double_tap(x, y)

    def double_click_view(self, class_name, text=None, index=0, duration=3):
        self.session(class_name=class_name, text=text, index=index).double_tap(duration)
        time.sleep(sleep_time)

    def force_touch_view(
        self,
        class_name=None,
        text=None,
        partial_text=None,
        index=0,
        pressure=1.0,
        duration=1.0,
    ):
        self.session(
            class_name=class_name, text=text, partial_text=partial_text, index=index
        ).force_touch(pressure, duration)
        time.sleep(sleep_time)

    def force_touch_point(self, x, y, pressure=1.0, duration=1.0):
        self.session(class_name="Window").force_touch_point(x, y, pressure, duration)
        time.sleep(sleep_time)

    def screenshot_view(self, filename, class_name=None, text=None, partial_text=None, index=0):
        self.session(
            class_name=class_name, text=text, partial_text=partial_text, index=index
        ).screenshot(filename)
        time.sleep(sleep_time)

    def scroll_height(self, direction="up", delt_height=0.5, duration=0.1):
        center_x = self.width / 2
        center_y = self.height / 2
        height = delt_height * self.height
        if direction == "up":
            self.swipe(
                center_x,
                center_y + height / 2,
                center_x,
                center_y - height / 2,
                duration=duration,
            )
        elif direction == "down":
            self.swipe(
                center_x,
                center_y - height / 2,
                center_x,
                center_y + height / 2,
                duration=duration,
            )
        else:
            raise Exception("error parameter, must be up or down")

    def scroll_then_click(self, text=None, partial_text=None, index=0, direction="visible"):
        if direction == "visible":
            if not self.elem(class_name="WebView").exists:
                scrollcount = 0
                scrollmaxcount = 15
                eletext = self.session(text=text, partial_text=partial_text, index=index)
                while not eletext.displayed and scrollcount < scrollmaxcount:
                    scrollcount += 1
                    self.scroll_height()
                time.sleep(1)
                if eletext.displayed:
                    eletext.click()
                    time.sleep(1)
                else:
                    raise RuntimeError("element not found")
            # webview用坐标判断:
            else:
                wv = self.elem(class_name="WebView").bounds
                x = wv.x + wv.width
                y = wv.y + wv.height
                scrollcount = 0
                scrollmaxcount = 15
                eletext = self.session(text=text, partial_text=partial_text, index=index)
                while (
                    not (eletext.bounds.x < x and eletext.bounds.y < y)
                    and scrollcount < scrollmaxcount
                ):
                    scrollcount += 1
                    self.scroll_height()
                time.sleep(1)
                try:
                    self.click_point(eletext.bounds.x + 10, eletext.bounds.y + 10)
                except Exception:
                    raise RuntimeError("element not found")
                time.sleep(1)
        else:
            self.session(text=text, partial_text=partial_text, index=index).scroll(
                direction
            ).click()

    def scroll_only(self, text=None, text_contains=None, index=0, direction="visible"):
        if direction == "visible":
            if not self.elem(class_name="WebView").exists:
                scrollcount = 0
                scrollmaxcount = 15
                eletext = self.session(text=text, partial_text=text_contains, index=index)
                while not eletext.displayed and scrollcount < scrollmaxcount:
                    scrollcount += 1
                    self.scroll_height()
            else:
                wv = self.elem(class_name="WebView").bounds
                x = wv.x + wv.width
                y = wv.y + wv.height
                scrollcount = 0
                scrollmaxcount = 15
                eletext = self.session(text=text, partial_text=text_contains, index=index)
                while (
                    not (eletext.bounds.x < x and eletext.bounds.y < y)
                    and scrollcount < scrollmaxcount
                ):
                    scrollcount += 1
                    self.scroll_height()
        else:
            self.session(text=text, partial_text=text_contains, index=index).scroll(direction)

    def enter_text(self, text, index=0, is_clear_text=False, class_name="TextView"):
        if is_clear_text:
            self.session(class_name=class_name, index=index).clear_text()
        self.session(class_name=class_name, index=index).set_text(text)
        time.sleep(sleep_time)

    def enter_field_text(self, text, index=0, is_clear_text=False):
        if is_clear_text:
            self.session(class_name="TextField", index=index).clear_text()
        self.session(class_name="TextField", index=index).set_text(text)
        time.sleep(sleep_time)

    def enter_search_text(self, text, index=0, is_clear_text=False):
        if is_clear_text:
            self.session(class_name="SearchField", index=index).clear_text()
        self.session(class_name="SearchField", index=index).set_text(text)
        time.sleep(sleep_time)

    @property
    def orientation(self):
        orient = self.session.orientation
        return orient

    def orientation_set(self, direction):
        self.session.orientationset(direction)

    def alert_accept(self):
        self.session.alert.accept()

    def alert_dismiss(self):
        self.session.alert.dismiss()

    def alert_text(self):
        return self.session.alert.text

    def alert_buttons(self):
        return self.session.alert.buttons()

    def alert_click_button(self, button_name):
        self.session.alert.click_button(button_name)

    def keyboard_dismiss(self):
        self.session.keyboard.dismiss()

    def elem(self, **kwargs):
        return self.session(**kwargs)

    def click_on_view(self, class_name, text=None, label=None, index=0):
        self.session(class_name=class_name, text=text, label=label, index=index).tap()
        time.sleep(sleep_time)

    def click_if_exists(self, class_name=None, text=None, index=0, timeout=10):
        self.elem(class_name=class_name, text=text, index=index).click_if_exists(timeout=timeout)

    def click_on_ui_view(self, view_elem):
        if view_elem is None:
            raise NoSuchElementError()
        x = view_elem.bounds.center[0]
        y = view_elem.bounds.center[1]
        self.click_point(x, y)
        time.sleep(sleep_time)

    def click_on_righttop(self, view_elem):
        if view_elem is None:
            raise NoSuchElementError()
        x = view_elem.bounds.left + (view_elem.bounds.right - view_elem.bounds.left) * 7 / 8
        y = view_elem.bounds.top + (view_elem.bounds.bottom - view_elem.bounds.top) / 8
        self.click_point(x, y)
        time.sleep(sleep_time)

    def search_text(self, text, class_name=None):
        time.sleep(sleep_time)

        if (
            self.elem(text=text, class_name=class_name).exists
            or self.elem(partial_text=text, class_name=class_name).exists
        ):
            return True

        return False

    def press_back(self):
        btns = self.elem(class_name="NavigationBar").subelems(class_name="Button")
        if btns.count > 1:
            if btns[0].bounds.x < btns[1].bounds.x:
                btns[0].tap()
            else:
                btns[1].tap()
        else:
            btns[0].tap()
        time.sleep(1)

    # view_elem and class_name
    def parent_view(self, child_elem, parent_class_name):
        count = self.elem(class_name=parent_class_name).count
        child_class_name = child_elem.class_name
        child_text = child_elem.text
        for index in range(count):
            parent_elem = self.elem(class_name=parent_class_name, index=index)
            if parent_elem.subelems(class_name=child_class_name, text=child_text).exists:
                return parent_elem

        raise RuntimeError("parent element not found")

    def zoom_in(self, text=None, index=0):
        self.session(class_name="Image", text=text, index=index).pinch("open")

    def zoom_out(self, text=None, index=0):
        self.session(class_name="Image", text=text, index=index).pinch("close")

    def get_index(self, class_name, text):
        elements = self.elem(class_name=class_name)
        for index in range(len(elements)):
            if elements[index].text == text:
                return index
        else:
            return -1


if __name__ == "__main__":
    c = WdaUI("http://localhost:8100", "com.tencent.qy.xin")
    c.session(class_name="Button", partial_text="讯录").click(timeout=1.0)



#!/usr/bin/env python3
# created 2020/3/13 by xiazeng
import json
import tgit
import requests
import os

proxies = {
    "http": "http://dev-proxy.oa.com:8080",
    "https": "http://dev-proxy.oa.com:8080",
}


def upload_issue(git_token, git_repo, case_filename, msg, assignee_names=None):
    if assignee_names is None:
        assignee_names = list()
    case_json = json.load(open(case_filename))
    git = tgit.TgitClient(tgit.PrivateKeyAuth(git_token))

    # step-1 创建issue
    code = "".join(case_json["source"]["code"])
    if case_json["success"]:
        return "caseissuccess"
    errors = [case_json["errors"], case_json["failures"]]
    exception_names = []
    for error in errors:
        if error:
            exception_names.append(error.strip().split("\n")[-1].split(":")[0].split(".")[-1])
    exceptions = "\n\n".join([e for e in errors if e])
    title = f"{'|'.join(exception_names)} {case_json['module']}#{case_json['case_name']}"
    description = f"{msg}\n\n代码：\n\n```\n{code}\n```\n\nexception:\n```\n{exceptions}\n```"
    # title='11'
    # description='22'
    assignee_ids = []
    for assignee_name in assignee_names:
        user_info = git.get_user(id=assignee_name)
        assignee_ids.append(str(user_info["id"]))

    issue = git.create_projects_issues(
        git_repo,
        title=title,
        labels=",".join(exception_names),
        description=description,
        assignee_ids=assignee_ids,
    )
    # step-2 贴上图片
    images_url = []
    result_path = os.path.dirname(case_filename)
    for p in case_json["screen_info"]:
        path = os.path.join(result_path, p["path"])
        r = git.upload_images(git_repo, path)
        images_url.append('<img src="%s" width="300">' % r["url"])
    issue_id = issue["id"]
    git.create_projects_issues_notes(git_repo, issue_id, body="".join(images_url))
    log_path = case_filename.replace("result", "log").replace("json", "log")

    # step-3 附上日志
    r = git.upload_file(git_repo, log_path, "'text/plain'")
    git.create_projects_issues_notes(git_repo, issue_id, body=f"{r['markdown']}\n")

    # log_lines = open(log_path).readlines()
    # content = '\n'.join([l for l in log_lines if "DEBUG" not in log_lines])
    # print(len(content))
    # 工蜂返回异常
    # git.create_projects_issues_notes(git_repo, issue_id, body=f"{r['markdown']}\n```\n{content}\n```")
    return f"https://git.code.tencent.com/minum/testupissue/issues/{issue['iid']}"


# if __name__ == "__main__":
# jf = "D:/weixin/miniumtest/miniprogram-demo-test/outputs/20200509192808/test_get_outerxml/20200509192837809186/test_get_outerxml.json"
# result = upload_issue("", 116331, jf, "ddd", ["xxx"])
# print(result)



#!/usr/local/bin/python3
# -*- coding: utf-8 -*-
"""
Author:         yopofeng
Filename:       utils.py
Create time:    2022/9/27 17:12
Description:

"""
from .utils import ProcessSafeEventLoop

event_loop = ProcessSafeEventLoop()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

xcui_element = [
    "Any",
    "Other",
    "Application",
    "Group",
    "Window",
    "Sheet",
    "Drawer",
    "Alert",
    "Dialog",
    "Button",
    "RadioButton",
    "RadioGroup",
    "CheckBox",
    "DisclosureTriangle",
    "PopUpButton",
    "ComboBox",
    "MenuButton",
    "ToolbarButton",
    "Popover",
    "Keyboard",
    "Key",
    "NavigationBar",
    "TabBar",
    "TabGroup",
    "Toolbar",
    "StatusBar",
    "Table",
    "TableRow",
    "TableColumn",
    "Outline",
    "OutlineRow",
    "Browser",
    "CollectionView",
    "Slider",
    "PageIndicator",
    "ProgressIndicator",
    "ActivityIndicator",
    "SegmentedControl",
    "Picker",
    "PickerWheel",
    "Switch",
    "Toggle",
    "Link",
    "Image",
    "Icon",
    "SearchField",
    "ScrollView",
    "ScrollBar",
    "StaticText",
    "TextField",
    "SecureTextField",
    "DatePicker",
    "TextView",
    "Menu",
    "MenuItem",
    "MenuBar",
    "MenuBarItem",
    "Map",
    "WebView",
    "IncrementArrow",
    "DecrementArrow",
    "Timeline",
    "RatingIndicator",
    "ValueIndicator",
    "SplitGroup",
    "Splitter",
    "RelevanceIndicator",
    "ColorWell",
    "HelpTag",
    "Matte",
    "DockItem",
    "Ruler",
    "RulerMarker",
    "Grid",
    "LevelIndicator",
    "Cell",
    "LayoutArea",
    "LayoutItem",
    "Handle",
    "Stepper",
    "Tab",
]



#!/usr/local/bin/python3
# -*- coding: utf-8 -*-
"""
Author:         lockerzhang
Filename:       minium_object.py
Create time:    2019/4/30 21:21
Description:

"""

from .minium_log import MonitorMetaClass
from .callback import Callback
from .connection import Command, Connection
from ...utils.injectjs import getInjectJsCode
from ...utils.utils import retry
from ...framework.exception import *
import subprocess
import time
import logging
import types

logger = logging.getLogger("minium")


class RetryableApi(MonitorMetaClass):
    """
    给attr_dict["__RETRY_API__"]中的接口(包括property.fget)添加retry修饰器
    __RETRY_CNT__: 接口重试次数, 默认2
    __RETRY_EXCEPTION__: 命中给定错误才重试, 默认(MiniTimeoutCauseByConnectionBreakError, MiniTimeoutCauseByClientOffline)
    """
    def __new__(mcs, cls_name, bases, attr_dict):
        if "__RETRY_API__" in attr_dict:
            if "__RETRY_CNT__" not in attr_dict:
                attr_dict["__RETRY_CNT__"] = 2
            RETRY_CNT = attr_dict["__RETRY_CNT__"]
            if "__RETRY_EXCEPTION__" not in attr_dict:
                attr_dict["__RETRY_EXCEPTION__"] = (MiniTimeoutCauseByConnectionBreakError, MiniTimeoutCauseByClientOffline)
            RETRY_EXCEPTION = attr_dict["__RETRY_EXCEPTION__"]
            for api in attr_dict["__RETRY_API__"]:
                if api.startswith("__"):  # private method
                    api = f"_{cls_name}{api}"
                if api not in attr_dict:
                    continue
                if isinstance(attr_dict[api], property):
                    if attr_dict[api].fget:
                        attr_dict[api] = property(retry(2, ValueError)(attr_dict[api].fget), attr_dict[api].fset, attr_dict[api].fdel, attr_dict[api].__doc__)
                if not isinstance(attr_dict[api], types.FunctionType):
                    continue
                attr_dict[api] = retry(RETRY_CNT, RETRY_EXCEPTION)(attr_dict[api])
        return super().__new__(mcs, cls_name, bases, attr_dict)


class MiniumObject(object, metaclass=MonitorMetaClass):
    _cant_use_interface = {}

    def __init__(self):
        self.logger = logger
        self.observers = {}
        self.connection: Connection = None

    def _do_shell(self, command, print_msg=True, input=b""):
        """
        执行 shell 语句
        :param command:
        :param print_msg:
        :return:
        """
        self.logger.info("de shell: %s" % command)
        p = subprocess.Popen(
            command,
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            stdin=subprocess.PIPE,
        )
        out, err = p.communicate(input)
        try:
            out = out.decode("utf8")
        except UnicodeDecodeError:
            out = out.decode("gbk")
        try:
            err = err.decode("utf8")
        except UnicodeDecodeError:
            err = err.decode("gbk")
        if print_msg:
            self.logger.info("err:\n%s" % err)
            self.logger.info("out:\n%s" % out)
        return out, err

    @classmethod
    def _can_i_use(cls, name):
        return not cls._cant_use_interface.get(name, False)

    @classmethod
    def _unset_interface(cls, name):
        cls._cant_use_interface[name] = True

    # protect method

    def _wait(self, func, timeout, interval=1):
        """
        等待直到`func`为true
        :func: callable function
        :timeout: timeout
        :interval: query step
        :return: bool
        """
        # func = lambda :True or False
        if not callable(func):
            return False
        s = time.time()
        timeout = timeout or interval
        while time.time() - s < timeout:
            if func():
                return True
            time.sleep(interval)
        return False

    def _call_wx_method(self, method, args=None, plugin_appid=None, sync=True, ignore_response=False):
        if args is None:
            args = []
        if not isinstance(args, list):
            if isinstance(args, str):
                # 如果是字符型参数，就可以不用管是否是 sync 方法，直接转数组传参即可
                args = [args]
            elif "Sync" in method:
                # 如果是 sync 方法，则需要从字典里面提取所有的 value 成为一个数组进行传参
                if isinstance(args, dict):
                    temp_args = list()
                    for key in args.keys():
                        temp_args.append(args[key])
                    args = temp_args
            else:
                # 异步方法的话无需管 args 是str 还是 dict，直接转成 list 即可
                args = [args]

        params = {"method": method, "args": args}
        if plugin_appid:
            params["pluginId"] = plugin_appid
        if not sync:
            return self.connection.send_async("App.callWxMethod", params, ignore_response=ignore_response)
        return self.connection.send("App.callWxMethod", params)

    def _evaluate(self, app_function: str, args=None, sync=False, desc=None):
        if not args:
            args = []
        if sync:
            return self.connection.send(
                Command(
                    "App.callFunction",
                    {"functionDeclaration": app_function, "args": args},
                    desc=desc,
                )
            )
        else:
            return self.connection.send_async(
                "App.callFunction", {"functionDeclaration": app_function, "args": args}
            )

    def _evaluate_js(
        self,
        filename,
        args=None,
        sync=True,
        default=None,
        code_format_info=None,
        mode=None,
        **kwargs,
    ):
        """
        运行 js 代码
        :param filename: {JS_PATH} 中 JS 文件名字（不需要后缀）
        :param code_format_info: JS 内容中需要进行格式化的信息, 如内容中包含 `%s` `%(arg)s` 等的可格式化信息
        :param args: 注入函数需要输入的参数列表
        :param sync: 同步执行函数
        :param mode: js mode: 仅支持es5还是都支持
        :param default: 同步结果返回默认值
        """
        if args is None:
            args = []
        ret = self._evaluate(
            getInjectJsCode(filename, format_info=code_format_info, mode=mode),
            args,
            sync=sync,
            **kwargs,
        )
        if sync:
            return ret.get("result", {}).get("result", default)
        return ret

    def _get_async_response(self, msg_id: str):
        return self.connection.get_aysnc_msg_return(msg_id)

    def _expose_function(self, name, binding_function):
        self.connection.register(name, binding_function)
        self.connection.send("App.addBinding", {"name": name})

    def _unregister(self, name, binding_function=None):
        self.connection.remove(name, binding_function)

    def _mock_wx_method(
        self,
        method,
        functionDeclaration: str = "",
        result=None,
        args=None,
        success=True,
        plugin_appid=None,
    ):
        if not args:
            args = []
        elif not isinstance(args, tuple):
            args = [args]
        callback_type = "ok" if success else "fail"
        params = {"method": method}
        if plugin_appid:
            params["pluginId"] = plugin_appid
        if functionDeclaration:
            params.update({"functionDeclaration": functionDeclaration, "args": args})
        elif isinstance(result, str):
            params["result"] = {
                "result": result,
                "errMsg": "%s:%s" % (method, callback_type),
            }
        elif isinstance(result, dict):
            params["result"] = result
        else:
            self.logger.warning("mock wx method accept str or dict result only")
            return
        return self.connection.send("App.mockWxMethod", params)

    def _mock_wx_js(
        self, method, filename, args=None, code_format_info=None, plugin_appid=None
    ):
        """
        mock方法中定义的替换函数直接返回一个promise, promise不需要reject, 自动化协议会根据回调的errmsg来判断回调success/fail
        """
        return self._mock_wx_method(
            method,
            getInjectJsCode(filename, format_info=code_format_info),
            args=args,
            plugin_appid=plugin_appid,
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Created by xiazeng on 2017/8/28
import airtest.aircv.template_matching
import airtest.aircv.aircv


def search(im_path, source_path):

    im_search = airtest.aircv.aircv.imread(im_path)
    im_source = airtest.aircv.aircv.imread(source_path)
    return airtest.aircv.template_matching.TemplateMatching(im_search, im_source).find_best_result()





# -*- coding: utf-8 -*-
"""
@author: 'xiazeng'
@created: 2016/10/24 
"""

KEYCODE_UNKNOWN = 0
"""* Key code constant: Soft Left key.
 * Usually situated below the display on phones and used as a multi-function
 * feature key for selecting a software defined function shown on the bottom left
 * of the display. """
KEYCODE_SOFT_LEFT = 1
"""* Key code constant: Soft Right key.
 * Usually situated below the display on phones and used as a multi-function
 * feature key for selecting a software defined function shown on the bottom right
 * of the display. """
KEYCODE_SOFT_RIGHT = 2
"""* Key code constant: Home key.
 * This key is handled by the framework and is never delivered to applications. """
KEYCODE_HOME = 3
"""* Key code constant: Back key. """
KEYCODE_BACK = 4
"""* Key code constant: Call key. """
KEYCODE_CALL = 5
"""* Key code constant: End Call key. """
KEYCODE_ENDCALL = 6
"""* Key code constant: '0' key. """
KEYCODE_0 = 7
"""* Key code constant: '1' key. """
KEYCODE_1 = 8
"""* Key code constant: '2' key. """
KEYCODE_2 = 9
"""* Key code constant: '3' key. """
KEYCODE_3 = 10
"""* Key code constant: '4' key. """
KEYCODE_4 = 11
"""* Key code constant: '5' key. """
KEYCODE_5 = 12
"""* Key code constant: '6' key. """
KEYCODE_6 = 13
"""* Key code constant: '7' key. """
KEYCODE_7 = 14
"""* Key code constant: '8' key. """
KEYCODE_8 = 15
"""* Key code constant: '9' key. """
KEYCODE_9 = 16
"""* Key code constant: '*' key. """
KEYCODE_STAR = 17
"""* Key code constant: '#' key. """
KEYCODE_POUND = 18
"""* Key code constant: Directional Pad Up key.
 * May also be synthesized from trackball motions. """
KEYCODE_DPAD_UP = 19
"""* Key code constant: Directional Pad Down key.
 * May also be synthesized from trackball motions. """
KEYCODE_DPAD_DOWN = 20
"""* Key code constant: Directional Pad Left key.
 * May also be synthesized from trackball motions. """
KEYCODE_DPAD_LEFT = 21
"""* Key code constant: Directional Pad Right key.
 * May also be synthesized from trackball motions. """
KEYCODE_DPAD_RIGHT = 22
"""* Key code constant: Directional Pad Center key.
 * May also be synthesized from trackball motions. """
KEYCODE_DPAD_CENTER = 23
"""* Key code constant: Volume Up key.
 * Adjusts the speaker volume up. """
KEYCODE_VOLUME_UP = 24
"""* Key code constant: Volume Down key.
 * Adjusts the speaker volume down. """
KEYCODE_VOLUME_DOWN = 25
"""* Key code constant: Power key. """
KEYCODE_POWER = 26
"""* Key code constant: Camera key.
 * Used to launch a camera application or take pictures. """
KEYCODE_CAMERA = 27
"""* Key code constant: Clear key. """
KEYCODE_CLEAR = 28
"""* Key code constant: 'A' key. """
KEYCODE_A = 29
"""* Key code constant: 'B' key. """
KEYCODE_B = 30
"""* Key code constant: 'C' key. """
KEYCODE_C = 31
"""* Key code constant: 'D' key. """
KEYCODE_D = 32
"""* Key code constant: 'E' key. """
KEYCODE_E = 33
"""* Key code constant: 'F' key. """
KEYCODE_F = 34
"""* Key code constant: 'G' key. """
KEYCODE_G = 35
"""* Key code constant: 'H' key. """
KEYCODE_H = 36
"""* Key code constant: 'I' key. """
KEYCODE_I = 37
"""* Key code constant: 'J' key. """
KEYCODE_J = 38
"""* Key code constant: 'K' key. """
KEYCODE_K = 39
"""* Key code constant: 'L' key. """
KEYCODE_L = 40
"""* Key code constant: 'M' key. """
KEYCODE_M = 41
"""* Key code constant: 'N' key. """
KEYCODE_N = 42
"""* Key code constant: 'O' key. """
KEYCODE_O = 43
"""* Key code constant: 'P' key. """
KEYCODE_P = 44
"""* Key code constant: 'Q' key. """
KEYCODE_Q = 45
"""* Key code constant: 'R' key. """
KEYCODE_R = 46
"""* Key code constant: 'S' key. """
KEYCODE_S = 47
"""* Key code constant: 'T' key. """
KEYCODE_T = 48
"""* Key code constant: 'U' key. """
KEYCODE_U = 49
"""* Key code constant: 'V' key. """
KEYCODE_V = 50
"""* Key code constant: 'W' key. """
KEYCODE_W = 51
"""* Key code constant: 'X' key. """
KEYCODE_X = 52
"""* Key code constant: 'Y' key. """
KEYCODE_Y = 53
"""* Key code constant: 'Z' key. """
KEYCODE_Z = 54
"""* Key code constant: ',' key. """
KEYCODE_COMMA = 55
"""* Key code constant: '.' key. """
KEYCODE_PERIOD = 56
"""* Key code constant: Left Alt modifier key. """
KEYCODE_ALT_LEFT = 57
"""* Key code constant: Right Alt modifier key. """
KEYCODE_ALT_RIGHT = 58
"""* Key code constant: Left Shift modifier key. """
KEYCODE_SHIFT_LEFT = 59
"""* Key code constant: Right Shift modifier key. """
KEYCODE_SHIFT_RIGHT = 60
"""* Key code constant: Tab key. """
KEYCODE_TAB = 61
"""* Key code constant: Space key. """
KEYCODE_SPACE = 62
"""* Key code constant: Symbol modifier key.
 * Used to enter alternate symbols. """
KEYCODE_SYM = 63
"""* Key code constant: Explorer special function key.
 * Used to launch a browser application. """
KEYCODE_EXPLORER = 64
"""* Key code constant: Envelope special function key.
 * Used to launch a mail application. """
KEYCODE_ENVELOPE = 65
"""* Key code constant: Enter key. """
KEYCODE_ENTER = 66
"""* Key code constant: Backspace key.
 * Deletes characters before the insertion point, unlike {@link #KEYCODE_FORWARD_DEL}. """
KEYCODE_DEL = 67
"""* Key code constant: '`' (backtick) key. """
KEYCODE_GRAVE = 68
"""* Key code constant: '-'. """
KEYCODE_MINUS = 69
"""* Key code constant: '=' key. """
KEYCODE_EQUALS = 70
"""* Key code constant: '[' key. """
KEYCODE_LEFT_BRACKET = 71
"""* Key code constant: ']' key. """
KEYCODE_RIGHT_BRACKET = 72
"""* Key code constant: '\' key. """
KEYCODE_BACKSLASH = 73
"""* Key code constant: '' key. """
KEYCODE_SEMICOLON = 74
"""* Key code constant: ''' (apostrophe) key. """
KEYCODE_APOSTROPHE = 75
"""* Key code constant: '/' key. """
KEYCODE_SLASH = 76
"""* Key code constant: '@' key. """
KEYCODE_AT = 77
"""* Key code constant: Number modifier key.
 * Used to enter numeric symbols.
 * This key is not Num Lock it is more like {@link #KEYCODE_ALT_LEFT} and is
 * interpreted as an ALT key by {@link android.text.method.MetaKeyKeyListener}. """
KEYCODE_NUM = 78
"""* Key code constant: Headset Hook key.
 * Used to hang up calls and stop media. """
KEYCODE_HEADSETHOOK = 79
"""* Key code constant: Camera Focus key.
 * Used to focus the camera. """
KEYCODE_FOCUS = 80 # * Camera * focus
"""* Key code constant: '+' key. """
KEYCODE_PLUS = 81
"""* Key code constant: Menu key. """
KEYCODE_MENU = 82
"""* Key code constant: Notification key. """
KEYCODE_NOTIFICATION = 83
"""* Key code constant: Search key. """
KEYCODE_SEARCH = 84
"""* Key code constant: Play/Pause media key. """
KEYCODE_MEDIA_PLAY_PAUSE = 85
"""* Key code constant: Stop media key. """
KEYCODE_MEDIA_STOP = 86
"""* Key code constant: Play Next media key. """
KEYCODE_MEDIA_NEXT = 87
"""* Key code constant: Play Previous media key. """
KEYCODE_MEDIA_PREVIOUS = 88
"""* Key code constant: Rewind media key. """
KEYCODE_MEDIA_REWIND = 89
"""* Key code constant: Fast Forward media key. """
KEYCODE_MEDIA_FAST_FORWARD = 90
"""* Key code constant: Mute key.
 * Mutes the microphone, unlike {@link #KEYCODE_VOLUME_MUTE}. """
KEYCODE_MUTE = 91
"""* Key code constant: Page Up key. """
KEYCODE_PAGE_UP = 92
"""* Key code constant: Page Down key. """
KEYCODE_PAGE_DOWN = 93
"""* Key code constant: Picture Symbols modifier key.
 * Used to switch symbol sets (Emoji, Kao-moji). """
KEYCODE_PICTSYMBOLS = 94 # switch
"""* Key code constant: Switch Charset modifier key.
 * Used to switch character sets (Kanji, Katakana). """
KEYCODE_SWITCH_CHARSET = 95 # switch
"""* Key code constant: A Button key.
 * On a game controller, the A button should be either the button labeled A
 * or the first button on the bottom row of controller buttons. """
KEYCODE_BUTTON_A = 96
"""* Key code constant: B Button key.
 * On a game controller, the B button should be either the button labeled B
 * or the second button on the bottom row of controller buttons. """
KEYCODE_BUTTON_B = 97
"""* Key code constant: C Button key.
 * On a game controller, the C button should be either the button labeled C
 * or the third button on the bottom row of controller buttons. """
KEYCODE_BUTTON_C = 98
"""* Key code constant: X Button key.
 * On a game controller, the X button should be either the button labeled X
 * or the first button on the upper row of controller buttons. """
KEYCODE_BUTTON_X = 99
"""* Key code constant: Y Button key.
 * On a game controller, the Y button should be either the button labeled Y
 * or the second button on the upper row of controller buttons. """
KEYCODE_BUTTON_Y = 100
"""* Key code constant: Z Button key.
 * On a game controller, the Z button should be either the button labeled Z
 * or the third button on the upper row of controller buttons. """
KEYCODE_BUTTON_Z = 101
"""* Key code constant: L1 Button key.
 * On a game controller, the L1 button should be either the button labeled L1 (or L)
 * or the top left trigger button. """
KEYCODE_BUTTON_L1 = 102
"""* Key code constant: R1 Button key.
 * On a game controller, the R1 button should be either the button labeled R1 (or R)
 * or the top right trigger button. """
KEYCODE_BUTTON_R1 = 103
"""* Key code constant: L2 Button key.
 * On a game controller, the L2 button should be either the button labeled L2
 * or the bottom left trigger button. """
KEYCODE_BUTTON_L2 = 104
"""* Key code constant: R2 Button key.
 * On a game controller, the R2 button should be either the button labeled R2
 * or the bottom right trigger button. """
KEYCODE_BUTTON_R2 = 105
"""* Key code constant: Left Thumb Button key.
 * On a game controller, the left thumb button indicates that the left (or only)
 * joystick is pressed. """
KEYCODE_BUTTON_THUMBL = 106
"""* Key code constant: Right Thumb Button key.
 * On a game controller, the right thumb button indicates that the right
 * joystick is pressed. """
KEYCODE_BUTTON_THUMBR = 107
"""* Key code constant: Start Button key.
 * On a game controller, the button labeled Start. """
KEYCODE_BUTTON_START = 108
"""* Key code constant: Select Button key.
 * On a game controller, the button labeled Select. """
KEYCODE_BUTTON_SELECT = 109
"""* Key code constant: Mode Button key.
 * On a game controller, the button labeled Mode. """
KEYCODE_BUTTON_MODE = 110
"""* Key code constant: Escape key. """
KEYCODE_ESCAPE = 111
"""* Key code constant: Forward Delete key.
 * Deletes characters ahead of the insertion point, unlike {@link #KEYCODE_DEL}. """
KEYCODE_FORWARD_DEL = 112
"""* Key code constant: Left Control modifier key. """
KEYCODE_CTRL_LEFT = 113
"""* Key code constant: Right Control modifier key. """
KEYCODE_CTRL_RIGHT = 114
"""* Key code constant: Caps Lock key. """
KEYCODE_CAPS_LOCK = 115
"""* Key code constant: Scroll Lock key. """
KEYCODE_SCROLL_LOCK = 116
"""* Key code constant: Left Meta modifier key. """
KEYCODE_META_LEFT = 117
"""* Key code constant: Right Meta modifier key. """
KEYCODE_META_RIGHT = 118
"""* Key code constant: Function modifier key. """
KEYCODE_FUNCTION = 119
"""* Key code constant: System Request / Print Screen key. """
KEYCODE_SYSRQ = 120
"""* Key code constant: Break / Pause key. """
KEYCODE_BREAK = 121
"""* Key code constant: Home Movement key.
 * Used for scrolling or moving the cursor around to the start of a line
 * or to the top of a list. """
KEYCODE_MOVE_HOME = 122
"""* Key code constant: End Movement key.
 * Used for scrolling or moving the cursor around to the end of a line
 * or to the bottom of a list. """
KEYCODE_MOVE_END = 123
"""* Key code constant: Insert key.
 * Toggles insert / overwrite edit mode. """
KEYCODE_INSERT = 124
"""* Key code constant: Forward key.
 * Navigates forward in the history stack.  Complement of {@link #KEYCODE_BACK}. """
KEYCODE_FORWARD = 125
"""* Key code constant: Play media key. """
KEYCODE_MEDIA_PLAY = 126
"""* Key code constant: Pause media key. """
KEYCODE_MEDIA_PAUSE = 127
"""* Key code constant: Close media key.
 * May be used to close a CD tray, for example. """
KEYCODE_MEDIA_CLOSE = 128
"""* Key code constant: Eject media key.
 * May be used to eject a CD tray, for example. """
KEYCODE_MEDIA_EJECT = 129
"""* Key code constant: Record media key. """
KEYCODE_MEDIA_RECORD = 130
"""* Key code constant: F1 key. """
KEYCODE_F1 = 131
"""* Key code constant: F2 key. """
KEYCODE_F2 = 132
"""* Key code constant: F3 key. """
KEYCODE_F3 = 133
"""* Key code constant: F4 key. """
KEYCODE_F4 = 134
"""* Key code constant: F5 key. """
KEYCODE_F5 = 135
"""* Key code constant: F6 key. """
KEYCODE_F6 = 136
"""* Key code constant: F7 key. """
KEYCODE_F7 = 137
"""* Key code constant: F8 key. """
KEYCODE_F8 = 138
"""* Key code constant: F9 key. """
KEYCODE_F9 = 139
"""* Key code constant: F10 key. """
KEYCODE_F10 = 140
"""* Key code constant: F11 key. """
KEYCODE_F11 = 141
"""* Key code constant: F12 key. """
KEYCODE_F12 = 142
"""* Key code constant: Num Lock key.
 * This is the Num Lock key it is different from {@link #KEYCODE_NUM}.
 * This key alters the behavior of other keys on the numeric keypad. """
KEYCODE_NUM_LOCK = 143
"""* Key code constant: Numeric keypad '0' key. """
KEYCODE_NUMPAD_0 = 144
"""* Key code constant: Numeric keypad '1' key. """
KEYCODE_NUMPAD_1 = 145
"""* Key code constant: Numeric keypad '2' key. """
KEYCODE_NUMPAD_2 = 146
"""* Key code constant: Numeric keypad '3' key. """
KEYCODE_NUMPAD_3 = 147
"""* Key code constant: Numeric keypad '4' key. """
KEYCODE_NUMPAD_4 = 148
"""* Key code constant: Numeric keypad '5' key. """
KEYCODE_NUMPAD_5 = 149
"""* Key code constant: Numeric keypad '6' key. """
KEYCODE_NUMPAD_6 = 150
"""* Key code constant: Numeric keypad '7' key. """
KEYCODE_NUMPAD_7 = 151
"""* Key code constant: Numeric keypad '8' key. """
KEYCODE_NUMPAD_8 = 152
"""* Key code constant: Numeric keypad '9' key. """
KEYCODE_NUMPAD_9 = 153
"""* Key code constant: Numeric keypad '/' key (for division). """
KEYCODE_NUMPAD_DIVIDE = 154
"""* Key code constant: Numeric keypad '*' key (for multiplication). """
KEYCODE_NUMPAD_MULTIPLY = 155
"""* Key code constant: Numeric keypad '-' key (for subtraction). """
KEYCODE_NUMPAD_SUBTRACT = 156
"""* Key code constant: Numeric keypad '+' key (for addition). """
KEYCODE_NUMPAD_ADD = 157
"""* Key code constant: Numeric keypad '.' key (for decimals or digit grouping). """
KEYCODE_NUMPAD_DOT = 158
"""* Key code constant: Numeric keypad ',' key (for decimals or digit grouping). """
KEYCODE_NUMPAD_COMMA = 159
"""* Key code constant: Numeric keypad Enter key. """
KEYCODE_NUMPAD_ENTER = 160
"""* Key code constant: Numeric keypad '=' key. """
KEYCODE_NUMPAD_EQUALS = 161
"""* Key code constant: Numeric keypad '(' key. """
KEYCODE_NUMPAD_LEFT_PAREN = 162
"""* Key code constant: Numeric keypad ')' key. """
KEYCODE_NUMPAD_RIGHT_PAREN = 163
"""* Key code constant: Volume Mute key.
 * Mutes the speaker, unlike {@link #KEYCODE_MUTE}.
 * This key should normally be implemented as a toggle such that the first press
 * mutes the speaker and the second press restores the original volume. """
KEYCODE_VOLUME_MUTE = 164
"""* Key code constant: Info key.
 * Common on TV remotes to show additional information related to what is
 * currently being viewed. """
KEYCODE_INFO = 165
"""* Key code constant: Channel up key.
 * On TV remotes, increments the television channel. """
KEYCODE_CHANNEL_UP = 166
"""* Key code constant: Channel down key.
 * On TV remotes, decrements the television channel. """
KEYCODE_CHANNEL_DOWN = 167
"""* Key code constant: Zoom in key. """
KEYCODE_ZOOM_IN = 168
"""* Key code constant: Zoom out key. """
KEYCODE_ZOOM_OUT = 169
"""* Key code constant: TV key.
 * On TV remotes, switches to viewing live TV. """
KEYCODE_TV = 170
"""* Key code constant: Window key.
 * On TV remotes, toggles picture-in-picture mode or other windowing functions. """
KEYCODE_WINDOW = 171
"""* Key code constant: Guide key.
 * On TV remotes, shows a programming guide. """
KEYCODE_GUIDE = 172
"""* Key code constant: DVR key.
 * On some TV remotes, switches to a DVR mode for recorded shows. """
KEYCODE_DVR = 173
"""* Key code constant: Bookmark key.
 * On some TV remotes, bookmarks content or web pages. """
KEYCODE_BOOKMARK = 174
"""* Key code constant: Toggle captions key.
 * Switches the mode for closed-captioning text, for example during television shows. """
KEYCODE_CAPTIONS = 175
"""* Key code constant: Settings key.
 * Starts the system settings activity. """
KEYCODE_SETTINGS = 176
"""* Key code constant: TV power key.
 * On TV remotes, toggles the power on a television screen. """
KEYCODE_TV_POWER = 177
"""* Key code constant: TV input key.
 * On TV remotes, switches the input on a television screen. """
KEYCODE_TV_INPUT = 178
"""* Key code constant: Set-top-box power key.
 * On TV remotes, toggles the power on an external Set-top-box. """
KEYCODE_STB_POWER = 179
"""* Key code constant: Set-top-box input key.
 * On TV remotes, switches the input mode on an external Set-top-box. """
KEYCODE_STB_INPUT = 180
"""* Key code constant: A/V Receiver power key.
 * On TV remotes, toggles the power on an external A/V Receiver. """
KEYCODE_AVR_POWER = 181
"""* Key code constant: A/V Receiver input key.
 * On TV remotes, switches the input mode on an external A/V Receiver. """
KEYCODE_AVR_INPUT = 182
"""* Key code constant: Red "programmable" key.
 * On TV remotes, acts as a contextual/programmable key. """
KEYCODE_PROG_RED = 183
"""* Key code constant: Green "programmable" key.
 * On TV remotes, actsas a contextual/programmable key. """
KEYCODE_PROG_GREEN = 184
"""* Key code constant: Yellow "programmable" key.
 * On TV remotes, acts as a contextual/programmable key. """
KEYCODE_PROG_YELLOW = 185
"""* Key code constant: Blue "programmable" key.
 * On TV remotes, acts as a contextual/programmable key. """
KEYCODE_PROG_BLUE = 186
"""* Key code constant: App switch key.
 * Should bring up the application switcher dialog. """
KEYCODE_APP_SWITCH = 187
"""* Key code constant: Generic Game Pad Button #1."""
KEYCODE_BUTTON_1 = 188
"""* Key code constant: Generic Game Pad Button #2."""
KEYCODE_BUTTON_2 = 189
"""* Key code constant: Generic Game Pad Button #3."""
KEYCODE_BUTTON_3 = 190
"""* Key code constant: Generic Game Pad Button #4."""
KEYCODE_BUTTON_4 = 191
"""* Key code constant: Generic Game Pad Button #5."""
KEYCODE_BUTTON_5 = 192
"""* Key code constant: Generic Game Pad Button #6."""
KEYCODE_BUTTON_6 = 193
"""* Key code constant: Generic Game Pad Button #7."""
KEYCODE_BUTTON_7 = 194
"""* Key code constant: Generic Game Pad Button #8."""
KEYCODE_BUTTON_8 = 195
"""* Key code constant: Generic Game Pad Button #9."""
KEYCODE_BUTTON_9 = 196
"""* Key code constant: Generic Game Pad Button #10."""
KEYCODE_BUTTON_10 = 197
"""* Key code constant: Generic Game Pad Button #11."""
KEYCODE_BUTTON_11 = 198
"""* Key code constant: Generic Game Pad Button #12."""
KEYCODE_BUTTON_12 = 199
"""* Key code constant: Generic Game Pad Button #13."""
KEYCODE_BUTTON_13 = 200
"""* Key code constant: Generic Game Pad Button #14."""
KEYCODE_BUTTON_14 = 201
"""* Key code constant: Generic Game Pad Button #15."""
KEYCODE_BUTTON_15 = 202
"""* Key code constant: Generic Game Pad Button #16."""
KEYCODE_BUTTON_16 = 203
"""* Key code constant: Language Switch key.
 * Toggles the current input language such as switching between English and Japanese on
 * a QWERTY keyboard.  On some devices, the same function may be performed by
 * pressing Shift+Spacebar. """
KEYCODE_LANGUAGE_SWITCH = 204
"""* Key code constant: Manner Mode key.
 * Toggles silent or vibrate mode on and off to make the device behave more politely
 * in certain settings such as on a crowded train.  On some devices, the key may only
 * operate when long-pressed. """
KEYCODE_MANNER_MODE = 205
"""* Key code constant: 3D Mode key.
 * Toggles the display between 2D and 3D mode. """
KEYCODE_3D_MODE = 206
"""* Key code constant: Contacts special function key.
 * Used to launch an address book application. """
KEYCODE_CONTACTS = 207
"""* Key code constant: Calendar special function key.
 * Used to launch a calendar application. """
KEYCODE_CALENDAR = 208
"""* Key code constant: Music special function key.
 * Used to launch a music player application. """
KEYCODE_MUSIC = 209
"""* Key code constant: Calculator special function key.
 * Used to launch a calculator application. """
KEYCODE_CALCULATOR = 210
"""* Key code constant: Japanese full-width / half-width key. """
KEYCODE_ZENKAKU_HANKAKU = 211
"""* Key code constant: Japanese alphanumeric key. """
KEYCODE_EISU = 212
"""* Key code constant: Japanese non-conversion key. """
KEYCODE_MUHENKAN = 213
"""* Key code constant: Japanese conversion key. """
KEYCODE_HENKAN = 214
"""* Key code constant: Japanese katakana / hiragana key. """
KEYCODE_KATAKANA_HIRAGANA = 215
"""* Key code constant: Japanese Yen key. """
KEYCODE_YEN = 216
"""* Key code constant: Japanese Ro key. """
KEYCODE_RO = 217
"""* Key code constant: Japanese kana key. """
KEYCODE_KANA = 218
"""* Key code constant: Assist key.
 * Launches the global assist activity.  Not delivered to applications. """
KEYCODE_ASSIST = 219
"""* Key code constant: Brightness Down key.
 * Adjusts the screen brightness down. """
KEYCODE_BRIGHTNESS_DOWN = 220
"""* Key code constant: Brightness Up key.
 * Adjusts the screen brightness up. """
KEYCODE_BRIGHTNESS_UP = 221
"""* Key code constant: Audio Track key.
 * Switches the audio tracks. """
KEYCODE_MEDIA_AUDIO_TRACK = 222
"""* Key code constant: Sleep key.
 * Puts the device to sleep.  Behaves somewhat like {@link #KEYCODE_POWER} but it
 * has no effect if the device is already asleep. """
KEYCODE_SLEEP = 223
"""* Key code constant: Wakeup key.
 * Wakes up the device.  Behaves somewhat like {@link #KEYCODE_POWER} but it
 * has no effect if the device is already awake. """
KEYCODE_WAKEUP = 224
"""* Key code constant: Pairing key.
 * Initiates peripheral pairing mode. Useful for pairing remote control
 * devices or game controllers, especially if no other input mode is
 * available. """
KEYCODE_PAIRING = 225
"""* Key code constant: Media Top Menu key.
 * Goes to the top of media menu. """
KEYCODE_MEDIA_TOP_MENU = 226
"""* Key code constant: '11' key. """
KEYCODE_11 = 227
"""* Key code constant: '12' key. """
KEYCODE_12 = 228
"""* Key code constant: Last Channel key.
 * Goes to the last viewed channel. """
KEYCODE_LAST_CHANNEL = 229
"""* Key code constant: TV data service key.
 * Displays data services like weather, sports. """
KEYCODE_TV_DATA_SERVICE = 230
"""* Key code constant: Voice Assist key.
 * Launches the global voice assist activity. Not delivered to applications. """
KEYCODE_VOICE_ASSIST = 231
"""* Key code constant: Radio key.
 * Toggles TV service / Radio service. """
KEYCODE_TV_RADIO_SERVICE = 232
"""* Key code constant: Teletext key.
 * Displays Teletext service. """
KEYCODE_TV_TELETEXT = 233
"""* Key code constant: Number entry key.
 * Initiates to enter multi-digit channel nubmber when each digit key is assigned
 * for selecting separate channel. Corresponds to Number Entry Mode (0x1D) of CEC
 * User Control Code. """
KEYCODE_TV_NUMBER_ENTRY = 234
"""* Key code constant: Analog Terrestrial key.
 * Switches to analog terrestrial broadcast service. """
KEYCODE_TV_TERRESTRIAL_ANALOG = 235
"""* Key code constant: Digital Terrestrial key.
 * Switches to digital terrestrial broadcast service. """
KEYCODE_TV_TERRESTRIAL_DIGITAL = 236
"""* Key code constant: Satellite key.
 * Switches to digital satellite broadcast service. """
KEYCODE_TV_SATELLITE = 237
"""* Key code constant: BS key.
 * Switches to BS digital satellite broadcasting service available in Japan. """
KEYCODE_TV_SATELLITE_BS = 238
"""* Key code constant: CS key.
 * Switches to CS digital satellite broadcasting service available in Japan. """
KEYCODE_TV_SATELLITE_CS = 239
"""* Key code constant: BS/CS key.
 * Toggles between BS and CS digital satellite services. """
KEYCODE_TV_SATELLITE_SERVICE = 240
"""* Key code constant: Toggle Network key.
 * Toggles selecting broacast services. """
KEYCODE_TV_NETWORK = 241
"""* Key code constant: Antenna/Cable key.
 * Toggles broadcast input source between antenna and cable. """
KEYCODE_TV_ANTENNA_CABLE = 242
"""* Key code constant: HDMI #1 key.
 * Switches to HDMI input #1. """
KEYCODE_TV_INPUT_HDMI_1 = 243
"""* Key code constant: HDMI #2 key.
 * Switches to HDMI input #2. """
KEYCODE_TV_INPUT_HDMI_2 = 244
"""* Key code constant: HDMI #3 key.
 * Switches to HDMI input #3. """
KEYCODE_TV_INPUT_HDMI_3 = 245
"""* Key code constant: HDMI #4 key.
 * Switches to HDMI input #4. """
KEYCODE_TV_INPUT_HDMI_4 = 246
"""* Key code constant: Composite #1 key.
 * Switches to composite video input #1. """
KEYCODE_TV_INPUT_COMPOSITE_1 = 247
"""* Key code constant: Composite #2 key.
 * Switches to composite video input #2. """
KEYCODE_TV_INPUT_COMPOSITE_2 = 248
"""* Key code constant: Component #1 key.
 * Switches to component video input #1. """
KEYCODE_TV_INPUT_COMPONENT_1 = 249
"""* Key code constant: Component #2 key.
 * Switches to component video input #2. """
KEYCODE_TV_INPUT_COMPONENT_2 = 250
"""* Key code constant: VGA #1 key.
 * Switches to VGA (analog RGB) input #1. """
KEYCODE_TV_INPUT_VGA_1 = 251
"""* Key code constant: Audio description key.
 * Toggles audio description off / on. """
KEYCODE_TV_AUDIO_DESCRIPTION = 252
"""* Key code constant: Audio description mixing volume up key.
 * Louden audio description volume as compared with normal audio volume. """
KEYCODE_TV_AUDIO_DESCRIPTION_MIX_UP = 253
"""* Key code constant: Audio description mixing volume down key.
 * Lessen audio description volume as compared with normal audio volume. """
KEYCODE_TV_AUDIO_DESCRIPTION_MIX_DOWN = 254
"""* Key code constant: Zoom mode key.
 * Changes Zoom mode (Normal, Full, Zoom, Wide-zoom, etc.) """
KEYCODE_TV_ZOOM_MODE = 255
"""* Key code constant: Contents menu key.
 * Goes to the title list. Corresponds to Contents Menu (0x0B) of CEC User Control
 * Code """
KEYCODE_TV_CONTENTS_MENU = 256
"""* Key code constant: Media context menu key.
 * Goes to the context menu of media contents. Corresponds to Media Context-sensitive
 * Menu (0x11) of CEC User Control Code. """
KEYCODE_TV_MEDIA_CONTEXT_MENU = 257
"""* Key code constant: Timer programming key.
 * Goes to the timer recording menu. Corresponds to Timer Programming (0x54) of
 * CEC User Control Code. """
KEYCODE_TV_TIMER_PROGRAMMING = 258
"""* Key code constant: Help key. """
KEYCODE_HELP = 259
"""* Key code constant: Navigate to previous key.
 * Goes backward by one item in an ordered collection of items. """
KEYCODE_NAVIGATE_PREVIOUS = 260
"""* Key code constant: Navigate to next key.
 * Advances to the next item in an ordered collection of items. """
KEYCODE_NAVIGATE_NEXT = 261
"""* Key code constant: Navigate in key.
 * Activates the item that currently has focus or expands to the next level of a navigation
 * hierarchy. """
KEYCODE_NAVIGATE_IN = 262
"""* Key code constant: Navigate out key.
 * Backs out one level of a navigation hierarchy or collapses the item that currently has
 * focus. """
KEYCODE_NAVIGATE_OUT = 263
"""* Key code constant: Primary stem key for Wear
 * Main power/reset button on watch. """
KEYCODE_STEM_PRIMARY = 264
"""* Key code constant: Generic stem key 1 for Wear """
KEYCODE_STEM_1 = 265
"""* Key code constant: Generic stem key 2 for Wear """
KEYCODE_STEM_2 = 266
"""* Key code constant: Generic stem key 3 for Wear """
KEYCODE_STEM_3 = 267
"""* Key code constant: Directional Pad Up-Left """
KEYCODE_DPAD_UP_LEFT = 268
"""* Key code constant: Directional Pad Down-Left """
KEYCODE_DPAD_DOWN_LEFT = 269
"""* Key code constant: Directional Pad Up-Right """
KEYCODE_DPAD_UP_RIGHT = 270
"""* Key code constant: Directional Pad Down-Right """
KEYCODE_DPAD_DOWN_RIGHT = 271
"""* Key code constant: Skip forward media key. """
KEYCODE_MEDIA_SKIP_FORWARD = 272
"""* Key code constant: Skip backward media key. """
KEYCODE_MEDIA_SKIP_BACKWARD = 273
"""* Key code constant: Step forward media key.
 * Steps media forward, one frame at a time. """
KEYCODE_MEDIA_STEP_FORWARD = 274
"""* Key code constant: Step backward media key.
 * Steps media backward, one frame at a time. """
KEYCODE_MEDIA_STEP_BACKWARD = 275
"""* Key code constant: put device to sleep unless a wakelock is held. """
KEYCODE_SOFT_SLEEP = 276
"""* Key code constant: Cut key. """
KEYCODE_CUT = 277
"""* Key code constant: Copy key. """
KEYCODE_COPY = 278
"""* Key code constant: Paste key. """
KEYCODE_PASTE = 279

MAX_KEYCODE = 84
"""*
 * {@link #getAction} value: the key has been pressed down.
 """
ACTION_DOWN = 0
"""*
 * {@link #getAction} value: the key has been released.
 """
ACTION_UP = 1
"""*
 * {@link #getAction} value: multiple duplicate key events have
 * occurred in a row, or a complex string is being delivered.  If the
 * key code is not {#link {@link #KEYCODE_UNKNOWN} then the
 * {#link {@link #getRepeatCount()} method returns the number of times
 * the given key code should be executed.
 * Otherwise, if the key code is {@link #KEYCODE_UNKNOWN}, then
 * this is a sequence of characters as returned by {@link #getCharacters}.
 """
ACTION_MULTIPLE = 2
"""*
 * SHIFT key locked in CAPS mode.
 * Reserved for use by {@link MetaKeyKeyListener} for a published constant in its API.
 * @hide
 """
META_CAP_LOCKED = 0x100
"""*
 * ALT key locked.
 * Reserved for use by {@link MetaKeyKeyListener} for a published constant in its API.
 * @hide
 """
META_ALT_LOCKED = 0x200
"""*
 * SYM key locked.
 * Reserved for use by {@link MetaKeyKeyListener} for a published constant in its API.
 * @hide
 """
META_SYM_LOCKED = 0x400
"""*
 * Text is in selection mode.
 * Reserved for use by {@link MetaKeyKeyListener} for a private unpublished constant
 * in its API that is currently being retained for legacy reasons.
 * @hide
 """
META_SELECTING = 0x800
"""*
 * <p>This mask is used to check whether one of the ALT meta keys is pressed.</p>
 *
 * @see #isAltPressed()
 * @see #getMetaState()
 * @see #KEYCODE_ALT_LEFT
 * @see #KEYCODE_ALT_RIGHT
 """
META_ALT_ON = 0x02
"""*
 * <p>This mask is used to check whether the left ALT meta key is pressed.</p>
 *
 * @see #isAltPressed()
 * @see #getMetaState()
 * @see #KEYCODE_ALT_LEFT
 """
META_ALT_LEFT_ON = 0x10
"""*
 * <p>This mask is used to check whether the right the ALT meta key is pressed.</p>
 *
 * @see #isAltPressed()
 * @see #getMetaState()
 * @see #KEYCODE_ALT_RIGHT
 """
META_ALT_RIGHT_ON = 0x20
"""*
 * <p>This mask is used to check whether one of the SHIFT meta keys is pressed.</p>
 *
 * @see #isShiftPressed()
 * @see #getMetaState()
 * @see #KEYCODE_SHIFT_LEFT
 * @see #KEYCODE_SHIFT_RIGHT
 """
META_SHIFT_ON = 0x1
"""*
 * <p>This mask is used to check whether the left SHIFT meta key is pressed.</p>
 *
 * @see #isShiftPressed()
 * @see #getMetaState()
 * @see #KEYCODE_SHIFT_LEFT
 """
META_SHIFT_LEFT_ON = 0x40
"""*
 * <p>This mask is used to check whether the right SHIFT meta key is pressed.</p>
 *
 * @see #isShiftPressed()
 * @see #getMetaState()
 * @see #KEYCODE_SHIFT_RIGHT
 """
META_SHIFT_RIGHT_ON = 0x80
"""*
 * <p>This mask is used to check whether the SYM meta key is pressed.</p>
 *
 * @see #isSymPressed()
 * @see #getMetaState()
 """
META_SYM_ON = 0x4
"""*
 * <p>This mask is used to check whether the FUNCTION meta key is pressed.</p>
 *
 * @see #isFunctionPressed()
 * @see #getMetaState()
 """
META_FUNCTION_ON = 0x8
"""*
 * <p>This mask is used to check whether one of the CTRL meta keys is pressed.</p>
 *
 * @see #isCtrlPressed()
 * @see #getMetaState()
 * @see #KEYCODE_CTRL_LEFT
 * @see #KEYCODE_CTRL_RIGHT
 """
META_CTRL_ON = 0x1000
"""*
 * <p>This mask is used to check whether the left CTRL meta key is pressed.</p>
 *
 * @see #isCtrlPressed()
 * @see #getMetaState()
 * @see #KEYCODE_CTRL_LEFT
 """
META_CTRL_LEFT_ON = 0x2000
"""*
 * <p>This mask is used to check whether the right CTRL meta key is pressed.</p>
 *
 * @see #isCtrlPressed()
 * @see #getMetaState()
 * @see #KEYCODE_CTRL_RIGHT
 """
META_CTRL_RIGHT_ON = 0x4000
"""*
 * <p>This mask is used to check whether one of the META meta keys is pressed.</p>
 *
 * @see #isMetaPressed()
 * @see #getMetaState()
 * @see #KEYCODE_META_LEFT
 * @see #KEYCODE_META_RIGHT
 """
META_META_ON = 0x10000
"""*
 * <p>This mask is used to check whether the left META meta key is pressed.</p>
 *
 * @see #isMetaPressed()
 * @see #getMetaState()
 * @see #KEYCODE_META_LEFT
 """
META_META_LEFT_ON = 0x20000
"""*
 * <p>This mask is used to check whether the right META meta key is pressed.</p>
 *
 * @see #isMetaPressed()
 * @see #getMetaState()
 * @see #KEYCODE_META_RIGHT
 """
META_META_RIGHT_ON = 0x40000
"""*
 * <p>This mask is used to check whether the CAPS LOCK meta key is on.</p>
 *
 * @see #isCapsLockOn()
 * @see #getMetaState()
 * @see #KEYCODE_CAPS_LOCK
 """
META_CAPS_LOCK_ON = 0x100000
"""*
 * <p>This mask is used to check whether the NUM LOCK meta key is on.</p>
 *
 * @see #isNumLockOn()
 * @see #getMetaState()
 * @see #KEYCODE_NUM_LOCK
 """
META_NUM_LOCK_ON = 0x200000
"""*
 * <p>This mask is used to check whether the SCROLL LOCK meta key is on.</p>
 *
 * @see #isScrollLockOn()
 * @see #getMetaState()
 * @see #KEYCODE_SCROLL_LOCK
 """
META_SCROLL_LOCK_ON = 0x400000

FLAG_WOKE_HERE = 0x1
"""*
 * This mask is set if the key event was generated by a software keyboard.
 """
FLAG_SOFT_KEYBOARD = 0x2
"""*
 * This mask is set if we don't want the key event to cause us to leave
 * touch mode.
 """
FLAG_KEEP_TOUCH_MODE = 0x4
"""*
 * This mask is set if an event was known to come from a trusted part
 * of the system.  That is, the event is known to come from the user,
 * and could not have been spoofed by a third party component.
 """
FLAG_FROM_SYSTEM = 0x8
"""*
 * This mask is used for compatibility, to identify enter keys that are
 * coming from an IME whose enter key has been auto-labelled "next" or
 * "done".  This allows TextView to dispatch these as normal enter keys
 * for old applications, but still do the appropriate action when
 * receiving them.
 """
FLAG_EDITOR_ACTION = 0x10
"""*
 * When associated with up key events, this indicates that the key press
 * has been canceled.  Typically this is used with virtual touch screen
 * keys, where the user can slide from the virtual key area on to the
 * display: in that case, the application will receive a canceled up
 * event and should not perform the action normally associated with the
 * key.  Note that for this to work, the application can not perform an
 * action for a key until it receives an up or the long press timeout has
 * expired.
 """
FLAG_CANCELED = 0x20
"""*
 * This key event was generated by a virtual (on-screen) hard key area.
 * Typically this is an area of the touchscreen, outside of the regular
 * display, dedicated to "hardware" buttons.
 """
FLAG_VIRTUAL_HARD_KEY = 0x40
"""*
 * This flag is set for the first key repeat that occurs after the
 * long press timeout.
 """
FLAG_LONG_PRESS = 0x80
"""*
 * Set when a key event has {@link #FLAG_CANCELED} set because a long
 * press action was executed while it was down.
 """
FLAG_CANCELED_LONG_PRESS = 0x100
"""*
 * Set for {@link #ACTION_UP} when this event's key code is still being
 * tracked from its initial down.  That is, somebody requested that tracking
 * started on the key down and a long press has not caused
 * the tracking to be canceled.
 """
FLAG_TRACKING = 0x200
"""*
 * Set when a key event has been synthesized to implement default behavior
 * for an event that the application did not handle.
 * Fallback key events are generated by unhandled trackball motions
 * (to emulate a directional keypad) and by certain unhandled key presses
 * that are declared in the key map (such as special function numeric keypad
 * keys when numlock is off).
 """
FLAG_FALLBACK = 0x400
"""*
 * Signifies that the key is being predispatched.
 * @hide
 """
FLAG_PREDISPATCH = 0x20000000
"""*
 * Private control to determine when an app is tracking a key sequence.
 * @hide
 """
FLAG_START_TRACKING = 0x40000000
"""*
 * Private flag that indicates when the system has detected that this key event
 * may be inconsistent with respect to the sequence of previously delivered key events,
 * such as when a key up event is sent but the key was not down.
 *
 * @hide
 * @see #isTainted
 * @see #setTainted
 """
FLAG_TAINTED = 0x80000000



#!/usr/bin/env python
# encoding:utf-8
# created:16/8/15
import json
import functools

__author__ = 'xiazeng'

import traceback
import xml.etree.ElementTree as ET
import xml.dom.minidom
import xml.dom
import re
import logging
import html

import at.utils.decorator

logger = logging.getLogger()


def parse_int(i):
    return int(i)


def parse_boolean(b):
    if b == "true":
        return True
    else:
        return False


def parse_bounds(bounds):
    m = re.match(r"\[(\d+),(\d+)\]\[(\d+),(\d+)\]", bounds)
    if m is None:
        return 0, 0, 0, 0
    return m.groups()


class ActivityProxy(object):
    def __init__(self, root_node, resgurad=None):
        self._document_node = root_node
        self._ui_views = [UiView(node, resgurad) for node in root_node.getElementsByTagName("node")]

    def find(self, **kwargs):
        for ui_view in self._ui_views:
            if ui_view.match(kwargs):
                return ui_view
        return None

    def find_all(self, **kwargs):
        ui_views = []
        for ui_view in self._ui_views:
            if ui_view.match(kwargs):
                ui_views.append(ui_view)
        return ui_views     # type: List[UiView]


class Rect(object):
    def __init__(self, x1, y1, x2, y2):
        self.left = x1
        self.top = y1
        self.right = x2
        self.bottom = y2
        self.center_x = (self.left + self.right) / 2
        self.center_y = (self.top + self.bottom) / 2
        self.width = self.right - self.left
        self.height = self.bottom - self.top

    def to_list(self):
        return [self.left, self.top, self.right, self.bottom]

    def contains(self, others):
        pass

    def __repr__(self):
        return "%d, %d, %d, %d" % (self.left, self.top, self.right, self.bottom)


class MOTION_EVENT:
    CLICK = 1
    LONG_CLICK = 2
    DOUBLE_CLICK = 3
    SCROLL_CLICK = 4
    SCROLL_DOWN = 5
    SCROLL_UP = 6
    SCROLL_LEFT = 7
    SCROLL_RIGHT = 8
    INPUT = 9


class BaseView(object):
    """
    控件的抽象
    """
    def get_rect(self):
        raise RuntimeError(u"Not Implementation")

    def surrport_action_ids(self):
        raise RuntimeError(u"Not Implementation")

    def perform(self):
        pass


class UiView(BaseView):
    def __init__(self, node: xml.dom.minidom.Element, resgurad=None):
        self._node = node
        self._node.ui_view = self  # 使得 node 可以得到 uiview 对象
        self.index = parse_int(self.g("index"))
        self.rg = resgurad
        self.raw_id = self.g("resource-id")
        if resgurad is not None:
            self.rid = resgurad.retrace_id(self.g("resource-id"))
        else:
            self.rid = self.g("resource-id")
        self.text = self.g("text")
        self.cls_name = self.g("class")
        self.package = self.g("package")
        self.content_desc = self.g("content-desc")
        self.display_id = self.g("display", None)
        self.checkable = parse_boolean(self.g("checkable"))
        self.checked = parse_boolean(self.g("checked"))
        self.clickable = parse_boolean(self.g("clickable"))
        self.enabled = parse_boolean(self.g("enabled"))
        self.focusable = parse_boolean(self.g("focusable"))
        self.focused = parse_boolean(self.g("focused"))
        self.scrollable = parse_boolean(self.g("scrollable"))
        self.long_clickable = parse_boolean(self.g("long-clickable"))
        self.password = self.g("password")
        self.selected = self.g("selected")
        self.bound_raw = self.g("bounds")
        self.bounds = parse_bounds(self.bound_raw)
        self.x1 = max(parse_int(self.bounds[0]), 0)
        self.y1 = max(parse_int(self.bounds[1]), 0)
        self.x2 = max(parse_int(self.bounds[2]), self.x1)
        self.y2 = max(parse_int(self.bounds[3]), self.y1)
        self.center_x = int((self.x1 + self.x2) / 2)
        self.center_y = int((self.y1 + self.y2) / 2)
        self.width = self.x2 - self.x1
        self.height = self.y2 - self.y1
        self.size = self.width * self.height
        self._cache = {}

    @classmethod
    def from_json(cls, json_data):
        pass

    def surrport_action_ids(self):
        action_ids = []
        if self.clickable:  action_ids.append(MOTION_EVENT.CLICK)
        if self.long_clickable: action_ids.append(MOTION_EVENT.LONG_CLICK)
        return action_ids

    def get_rect(self):
        return Rect(self.x1, self.y1, self.x2, self.y2)

    def group_name(self):
        """
        用于分组的标识
        """
        return u", ".join(
            [u"package=" + self.package, u"class=" + self.cls_name, u"resource-id=" + self.rid, u"text=" + self.text,
             u"content-desc=" + self.content_desc, u"size=%d" % self.size])

    @property
    def parent(self):
        parent_node = self._node.parentNode
        try:
            return parent_node.ui_view
        except:
            return None

    # @functools.lru_cache(1024)
    def get_xpath(self, ignore_text=False, ignore_index=False):
        """获取xpath
        /div[@id='id-74385'][@class='guest clearfix']
        """
        cache_key = f"ignore_text={ignore_text},ignore_index={ignore_index}"
        if cache_key in self._cache:
            return self._cache[cache_key]
        parent = self
        xpath = ""
        while parent:
            current_path = f"/{parent._node.tagName}"
            attr = {
                'class': parent.cls_name,
                'enabled': parent.enabled
            }
            if not ignore_index:
                current_path += f'[{parent.index}]'

            if not ignore_text and parent.text:
                attr['text'] = parent.text
            if parent.content_desc:
                attr['content-desc'] = parent.content_desc
            if parent.rid:
                attr['resource-id'] = parent.rid
            for key, value in attr.items():
                current_path += f"[@{key}='{value}']"
            xpath = current_path + xpath
            parent = parent.parent
        self._cache[cache_key] = xpath
        return xpath

    def get_equivalence_views(self):
        """
        获取中心点一样的节点列表，对这些元素进行点击的效果是一样的
        :return: 返回parent列表和child列表，两个列表都是按照亲疏关系排序
        """
        eq_parents = []
        eq_children = []
        parent = self.parent
        while parent:
            if parent.size == self.size and parent.center_x == self.center_x and self.center_y == parent.center_y:
                logger.debug("+parent:%s", parent)
                eq_parents.insert(0, parent)
                parent = parent.parent
            else:
                break

        for child_view in self.get_descendants():
            if child_view != self and child_view.center_x == self.center_x and child_view.center_y == self.center_y:
                logger.debug("+child:%s", child_view)
                eq_children.append(child_view)
        return eq_parents, eq_children

    def get_ancestors(self):
        views = []
        parent = self.parent
        while parent:
            views.append(parent)
            parent = parent.parent
        return views

    def is_leaf(self):
        return not self._node.hasChildNodes()

    def get_children(self):
        return [UiView(child_node, self.rg) for child_node in self._node.childNodes if
                child_node.nodeType == xml.dom.Node.ELEMENT_NODE]

    def get_descendants(self):
        views = [self, ]
        for view in self.get_children():
            views += view.get_descendants()
        return views

    def first_text(self):
        if self.all_texts():
            return self.all_texts()[0]
        return ""

    def all_texts(self):
        return [v.text for v in self.get_descendants() if v.text]

    def find(self, **kwargs):
        if self.match(kwargs):
            return self
        for view in self.get_children():
            ret = view.find(**kwargs)
            if ret:
                return ret
        return None

    def sibling(self, **kwargs):
        if self.parent:
            for ui_view in self.parent.get_children():
                if ui_view != self and ui_view.match(kwargs):
                    return ui_view
        return None

    def __eq__(self, obj):
        if obj is None or not isinstance(obj, UiView):
            return False
        ret = self.index == obj.index and \
            self.cls_name == obj.cls_name and \
            self.x1 == obj.x1 and\
            self.y1 == obj.y1 and \
            self.x2 == obj.x2 and \
            self.y2 == obj.y2 and \
            self.rid == obj.rid and \
            self.content_desc == obj.content_desc and \
            (self.text == obj.text if self.cls_name.endswith(".EditText") else True)
        return ret

    def __ne__(self, other):
        if self == other:
            return False
        return True

    def __hash__(self):
        return hash(self.__repr__())

    def unique_key(self):
        return self._node.toxml()

    def contains(self, view):
        if self == view:
            return False
        if view is None:
            return False
        if view.x1 == self.x1 and view.y1 == self.y1:
            if self.x2 == view.x2 and self.y2 == view.y2:
                return False

        if self.x2 >= view.x1 >= self.x1 and self.y2 >= view.y1 >= self.y1:
            if self.x2 >= view.x2 >= self.x1 and self.y2 >= view.y2 >= self.y1:
                return True
        return False

    def match(self, selector):
        if selector is None:
            return True
        if not isinstance(selector, dict):
            logger.error("selector not dict, is %s" % type(selector))
            return False
        for key, value in selector.items():
            if key == "rid":
                value = value.split("/")[-1]
            if not value:
                if getattr(self, key):
                    break
            elif isinstance(value, int):
                if value != getattr(self, key, -1):
                    break
            elif value not in getattr(self, key, ""):
                break
        else:
            return True
        return False

    def g(self, attr, default_value=None):
        try:
            value = self._node.getAttribute(attr)
        except:
            return default_value
        if isinstance(value, str):
            value = value.strip()
        return value

    def to_dict(self):
        return {
            "rid": self.rid,
            "resource-id": self.rid,
            "index": self.index,
            "class": self.cls_name,
            "text": self.text,
            "enabled": self.enabled,
            "package": self.package,
            "content-desc": self.content_desc,
            "clickable": self.clickable,
            "long_clickable": self.long_clickable,
            "scrollable": self.scrollable,
            "checked": self.checked,
            "rect": self.get_rect().to_list(),
            # "id": hash(self)
        }

    def dump_to_jsons(self):
        current_node = self.to_dict()
        current_node['nodes'] = []
        for child_node in self.get_children():
            current_node['nodes'].append(child_node.dump_to_jsons())
        return current_node

    def dump_to_json(self, filename):
        """
        转换成json文件，包含孩子节点
        :param filename:
        :return:
        """
        json_data = self.dump_to_jsons()
        with open(filename, "w") as f:
            json.dump(json_data, f)

    def __repr__(self):
        return ",".join([self.cls_name, self.content_desc, self.rid, self.package, self.first_text()])

    def __getstate__(self):
        return self.__dict__

    def __setstate__(self, d):
        self.__dict__.update(d)

    def __str__(self):
        return u", ".join([u"package=" + self.package, u"class=" + self.cls_name, u"enabled=%s" % self.enabled,
                           u"resource-id=" + self.rid, u"text=" + self.text, u"content-desc=" + self.content_desc,
                           u"checked=%s" % self.checked, u"bounds=" + self.bound_raw, u"index=%s" % self.index])


def ui_views_contains_texts(ui_views, *texts):
    text_dict = dict((text, 0) for text in texts)
    for ui_view in ui_views:
        for text in text_dict:
            if text in ui_view.text:
                text_dict[text] = 1
    for k, v in text_dict.items():
        if v == 0:
            return False
    return True


def ui_views_match(ui_views, *sub_views):
    """
    todo: 重构
    :param ui_views: 
    :param sub_views: 
    :return: 
    """
    sub_set_views = set(sub_views)
    for ui_view in ui_views:
        for sub_view in sub_views:
            if sub_view in sub_set_views:
                sub_set_views.remove(sub_view)
    if sub_set_views:
        return False
    return True


def view_filter(views, selector):
    result_views = []
    for view in views:
        if view.match(selector):
            result_views.append(view)
    return result_views


def get_view(views, selector, instance=0):
    hit_views = get_views(views, selector)
    if len(hit_views) <= abs(instance):
        return None
    return hit_views[instance]  # type: UiView


def get_views(views, selector):
    rets = []
    try:
        iter(views)
    except TypeError:
        logger.info("not iterable")
    else:
        for view in views:
            if view.match(selector):
                rets.append(view)
    return rets


def get_view_parents(views, target):
    result_views = []
    for view in views:
        if view.contains(target):
            result_views.append(view)
    return result_views


def get_child(views, target, selector, instance=0):
    rets = get_children(views, target, selector)
    if len(rets) > abs(instance):
        return rets[instance]
    return None


def get_children(views, target, selector):
    rets = []
    for view in views:
        if target.contains(view):
            if view.match(selector):
                rets.append(view)
    return rets


def window_dump_parse(filename):
    dom = xml.dom.minidom.parse(filename)
    root = dom.documentElement
    nodes = root.getElementsByTagName("node")
    return [UiView(child) for child in nodes]


def remove_kinda_window(xml_content):
    """
    支付界面XML有很多kinda_main_container，只保留最后一个，其他的删除
    """
    tree = ET.fromstring(xml_content)
    kinda_list = []
    for parent in tree.iter("*"):
        for child in parent:
            rid = child.attrib.get('resource-id')
            if rid == 'com.tencent.mm:id/kinda_main_container':
                kinda_list.append((child, parent))
                logger.debug(child.attrib)
                break
    if len(kinda_list) > 1:
        for kind, parent in kinda_list[:-1]:
            parent.remove(kind)
    return ET.tostring(tree, 'utf-8').decode('utf-8')


def window_dump_parse_str(xml_content, resgurad=None):
    "根据XML实例化成View对象，平铺返回"
    nodes = []
    try:
        xml_content = xml_content.encode("utf-8")
        xml_content = remove_kinda_window(xml_content)
        dom = xml.dom.minidom.parseString(xml_content)  # 这里有可能出错, 因为传入的s有问题
        root = dom.documentElement
        nodes = root.getElementsByTagName("node")
    except Exception:
        logger.error(xml_content)
        logger.exception("parse xml failed")
    return [UiView(child, resgurad) for child in nodes]


def json_to_xml_content(node, root=None):
    if root is None:
        root = ET.fromstring(f"""
        <hierarchy rotation="0">
        </hierarchy>
        """)
    rect = node['rect']
    if isinstance(rect, list):
        bounds = f"[{rect[0]},{rect[1]}][{rect[2]},{rect[3]}]"
    elif isinstance(rect, dict):
        bounds = f"[{rect['x']},{rect['y']}][{rect['x'] + rect['w']},{rect['y'] + rect['h']}]"
    text = html.escape(node['text'])
    desc = html.escape(node['content-desc'])
    elem = ET.fromstring(f"""
        <node index="{node['index']}" text="{text}" class="{node['class']}" package="{node['package']}" content-desc="{desc}"
          checkable="false" checked="false" clickable="false" enabled="true" focusable="false" focused="false"
          scrollable="false" long-clickable="false" password="false" selected="false" bounds="{bounds}"
          resource-id="{node['resource-id']}"></node>
        """)
    root.append(elem)
    for sub_node in node.get('nodes', list()):
        json_to_xml_content(sub_node, elem)

    xml_content = ET.tostring(root, 'unicode')
    return xml_content


def json_2_ui_views(node):
    xml_content = json_to_xml_content(node)
    return window_dump_parse_str(xml_content)


def window_dump_2_activity_proxy(xml_content, resgurad=None):
    try:
        xml_content = xml_content.encode("utf-8")
        xml_content = remove_kinda_window(xml_content)
        dom = xml.dom.minidom.parseString(xml_content)  # 这里有可能出错, 因为传入的s有问题
        root_node = dom.documentElement
        return ActivityProxy(root_node, resgurad)
    except Exception:
        logger.exception("parse xml failed")
    return None


def ui_views_same_rate(ui_views, other_views):
    if not other_views or not ui_views:
        return 0
    eq_num = 0
    for v in ui_views:
        for other_view in other_views:
            if other_view == v:
                eq_num += 1
                break
    return eq_num * 1.0 / len(other_views) if other_views else 0


def find_ui_view_by_element(ui_views, elem):
    for ui_view in ui_views:
        if elem.match_ui_view(ui_view):
            return ui_view
    return None


if __name__ == '__main__':
    vs = window_dump_parse("window_dump.xml")
    import pickle

    pickle.dump(vs, open('test.pickle', 'w'), 2)
    ss = pickle.load(open('test.pickle', 'rb'))
    print(1)
    # pickle.dump(vs, open('test.pickle', 'w'), 2)



# -*- coding: utf-8 -*-
#!/usr/bin/env python3

"""
Author:         xiazeng
Create time:    2020/9/21
Description:    

"""
import hashlib
import inspect
import json
import logging
import os.path
import uuid
import typing

import requests
from case_repair_sdk import default_trace

logger = logging.getLogger()

G_SERVER_HOST = "http://api.mt.woa.com/epuiat/"
if os.getenv("UIAT_HOST"):
    G_SERVER_HOST = os.getenv("UIAT_HOST")

DEBUG = True


def set_debug():
    global DEBUG, G_SERVER_HOST
    DEBUG = True
    G_SERVER_HOST = "http://9.134.52.250:30006/"


class RepairError(Exception):
    pass


class RecordError(Exception):
    pass


class LocatorNotFound(RecordError):
    pass


class RestartError(RepairError):
    pass


class ExitError(RepairError):
    pass


class CheckError(RepairError):
    pass


class InternalError(RepairError):
    pass


class RepairAction:
    CLICK = "click"
    LONG_CLICK = "long_click"
    RESTART = "restart"
    EXIT = "exit"


def check_response_data(r):
    if r.status_code != 200:
        logger.error(r.text)
    r.raise_for_status()
    # logger.debug(r.text.encode('utf-8'))
    res = json.loads(r.text)
    if res["rtn"] != 0:
        logger.error(r.text.encode('utf-8'))
        raise InternalError(res.get('reason') or res.get('msg'))
    repair_action = res.get("repair_action")
    if repair_action == RepairAction.EXIT:
        raise ExitError()
    elif repair_action == RepairAction.RESTART:
        raise RestartError()
    return res


class CaseRepairSdK:
    def __init__(self, test_id, app_id):
        self.test_id = test_id
        self.app_id = app_id
        self.trace_cache = dict()
        self._enable_trace_line = False
        self._cb_get_trace_data: typing.Callable[[bool, dict], typing.Tuple[str, dict]] = default_trace.get_trace_data
        self._cb_get_statement: typing.Callable[[], str] = default_trace.get_statement

    def set_get_trace_data_callback(self, cb: typing.Callable[[bool, dict], typing.Tuple[str, dict]]):
        """
        自定义获取trace
        :param cb:获取trace的回调函数
        :return:
        """
        self._cb_get_trace_data = cb

    def set_get_statement(self, cb: typing.Callable[[], str]):
        self._cb_get_statement = db

    def config_enable_trace_line(self, enabled):
        """计算trace_id是否计算行号"""
        self._enable_trace_line = enabled

    def _request(self, method, path, params=None, data=None, **kwargs):
        if params is None:
            params = {}
        params.update({
            "test_id": self.test_id,
            "app_id": self.app_id
        })
        session = requests.session()
        # session.trust_env = False
        url = G_SERVER_HOST + path
        logger.info("%s, params:%s", url, params)
        if DEBUG:
            json.dump({
                "url": url,
                "params": params,
                "data": data,
                "method": method
            }, open("debug_request.json", "w"))
        if method == 'post':
            http_res = session.post(url, params=params, data=data, **kwargs)
        else:
            http_res = session.get(url, params=params, **kwargs)
        session.close()
        logger.debug("%s, status:%s, text:%s", url, http_res.status_code, http_res.text)
        return check_response_data(http_res)

    def _get(self, path, params=None, **kwargs):
        return self._request('get', path, params, **kwargs)

    def _post(self, path, params=None, data=None, **kwargs):
        return self._request('post', path, params, data, **kwargs)

    def remove_test_data(self):
        """删除服务器上的数据"""
        self._get("repair/remove_test_data", params={"test_id": self.test_id})

    def generate_seq_id(self):
        return str(uuid.uuid4())

    def update_ui_seq(self, seq_id, seq_info):
        """
        录制模式/修复模式都需要调用。更新用例结果
        """
        seq_info['seq_id'] = seq_id
        seq_info['test_id'] = self.test_id
        return self._post("repair/update_ui_seq", data=json.dumps(seq_info))

    def upload_test_step(self, seq_name, page_name, seq_id, before_xml, after_xml, before_img_base64,
                         after_img_base64, operation, after_page_name=None, trace_id=None, traces=None):
        """
        录制模式调用。上传用例的UI操作（如单击，输入，长按等）的前后信息
        :param seq_name: 用例的独一无二的名称
        :param page_name: 当前界面页面的名称，Android推荐用activity的名称
        :param seq_id: 用例的唯一ID
        :param before_xml: UI操作之前的xml
        :param after_xml: UI操作之后的xml
        :param before_img_base64: UI操作之前的图片
        :param after_img_base64: UI操作之后的图片
        :param operation: 具体的执行动作，格式:
            {
                "action": "xxx",
                "args": [],
                "kwargs": {},
                "widget": {
                    "y": 1256,
                    "x": 541,
                    "w": 431,
                    "h": 151
                }
            }
        :param trace_id: 保证每个UI操作唯一，默认根据函数调用关系计算出hash值
        :param traces:
        :return:
        """
        if not trace_id:
            trace_id, trace = self._cb_get_trace_data(self._enable_trace_line, self.trace_cache)
        data = {
            "trace": trace,
            "before_xml": before_xml,
            "after_xml": after_xml,
            "before_img": before_img_base64,
            "after_img": after_img_base64,
            "operation": operation,
            "after_page_name": after_page_name,
            "statement": self._cb_get_statement()
        }
        params = {
            "test_id": self.test_id,
            "seq_name": seq_name,
            "page_name": page_name,
            "trace_id": trace_id,
            "seq_id": seq_id
        }
        return self._post("repair/save_test_step", params, json.dumps(data))

    def obtain_candidate(self, seq_name, page_name, seq_id, before_xml, before_img_base64, operation, trace_id=None,
                         trace=None):
        if not trace_id:
            trace_id, trace = self._cb_get_trace_data(self._enable_trace_line, self.trace_cache)
        params = {
            "test_id": self.test_id,
            "seq_name": seq_name,
            "page_name": page_name,
            "trace_id": trace_id,
            "seq_id": seq_id,
        }

        data = {
            "trace": trace,
            "before_xml": before_xml,
            "before_img": before_img_base64,
            "operation": operation,
            "statement": self._cb_get_statement()
        }
        res = self._post("repair/obtain_candidate", params, json.dumps(data))

        return res, trace_id

    def check_repair(self, seq_name, page_name, seq_id, after_xml, after_img_base64, trace_id, operation):
        """检查修复之后的结果"""
        params = {
            "test_id": self.test_id,
            "seq_name": seq_name,
            "page_name": page_name,
            "trace_id": trace_id,
            "seq_id": seq_id
        }

        data = {
            "operation": operation,
            "after_xml": after_xml,
            "after_img": after_img_base64
        }

        res = self._post("repair/check_repair", params, data=json.dumps(data))
        return res

    def start_repair_test(self, test_id, test_info):
        """开始测试的时候"""
        self._post("repair/start_repair_test", params={'test_id': test_id}, data=json.dumps(test_info))



#!/usr/bin/env python3
import ddt
import inspect
import re

__all__ = ["ddt_class", "ddt_case", "ddt_unpack", "ddt_data"]
index_len = 5


class Data(object):
    def __init__(self, data, name="") -> None:
        self.data = data
        self.__name__ = name


def mk_test_name(testname, value, index=0, name="", name_fmt="%(name)s_%(value)s"):
    index = "{0:0{1}}".format(index + 1, index_len)
    if name_fmt.find("%(index)s") >= 0:
        name_fmt = "%(testname)s_" + name_fmt
    else:
        name_fmt = "%(testname)s_%(index)s_" + name_fmt
    if not ddt.is_trivial(value):  # 不支持序列化类型
        name_fmt = re.sub(r"_?%\(value\)s", "", name_fmt)
    if not name:
        name_fmt = re.sub(r"_?%\(name\)s", "", name_fmt)
    try:
        value = str(value)
    except UnicodeEncodeError:
        # fallback for python2
        value = value.encode("ascii", "backslashreplace")
    test_name = name_fmt % {
        "testname": testname,
        "name": name,
        "index": index,
        "value": value,
    }
    return re.sub(r"\W|^(?=\d)", "_", test_name)[:200]  # 系统文件命名长度有限制，不宜过长


"""
rewrite ddt:
1. add definition for test name, Default is: "%(name)s_%(value)s"
    - %(index)s     needed
    - %(value)s     optional
    - %(name)s      optional
    e.g.
    @ddt_class(testNameFormat="%(name)s")
    class a:
        @ddt_case(ddt_data("abc", "name1"))
        def test(self):
            pass
    -> 
    class a:
        def test_1_name1(self):
            pass
"""


def ddt_class(arg=None, **kwargs):
    fmt_test_name = kwargs.get("testNameFormat", "%(index)s_%(name)s_%(value)s")

    def wrapper(cls):
        for name, func in list(cls.__dict__.items()):
            if hasattr(func, ddt.DATA_ATTR) and getattr(func, ddt.DATA_ATTR):
                for i, v in enumerate(getattr(func, ddt.DATA_ATTR)):
                    _name = ""
                    if isinstance(v, Data):
                        _name = getattr(v, "__name__", "")
                        v = v.data
                    test_name = mk_test_name(name, v, i, _name, fmt_test_name)
                    test_data_docstring = ddt._get_test_data_docstring(func, v)
                    if hasattr(func, ddt.UNPACK_ATTR):
                        if isinstance(v, (tuple, list)):
                            ddt.add_test(cls, test_name, test_data_docstring, func, *v)
                        elif isinstance(v, dict):
                            # unpack dictionary
                            ddt.add_test(cls, test_name, test_data_docstring, func, **v)
                    else:
                        ddt.add_test(cls, test_name, test_data_docstring, func, v)
                delattr(cls, name)
            elif hasattr(func, ddt.FILE_ATTR) and getattr(func, ddt.FILE_ATTR):
                file_attr = getattr(func, ddt.FILE_ATTR)
                ddt.process_file_data(cls, name, func, file_attr)
                delattr(cls, name)
        return cls

    # ``arg`` is the unittest's test class when decorating with ``@ddt`` while
    # it is ``None`` when decorating a test class with ``@ddt(k=v)``.
    return wrapper(arg) if inspect.isclass(arg) else wrapper


def ddt_data(data, name=""):
    return Data(data, name)


def ddt_case(*values):
    global index_len
    index_len = len(str(len(values)))
    return ddt.data(*values)


def ddt_unpack(func):
    return ddt.unpack(func)



# encoding=utf-8
import json


class TabDescription(object):
    def __init__(self, desc_text):
        self.desc_text = desc_text
        desc = {}
        try:
            desc = json.loads(desc_text)
        except ValueError:
            pass
        self.attached = desc.get("attached", None)
        self.empty = desc.get("empty", None)
        self.height = desc.get("height", None)
        self.screenX = desc.get("screenX", None)
        self.screenY = desc.get("screenY", None)
        self.visible = desc.get("visible", None)
        self.width = desc.get("width", None)
        self.title = desc.get("title", None)

    def __repr__(self):
        return self.desc_text



#!/usr/local/bin/python3
# -*- coding: utf-8 -*-
'''
Author: yopofeng
Date: 2021-09-03 21:24:30
LastEditTime: 2021-10-19 11:32:54
LastEditors: yopofeng
Description: 
'''


from .wdaUI import *
from .webDriverTool import *
from ..wda import (
    WDAError,
    WDAElementNotFoundError,
    WDARequestError,
    WDAEmptyResponseError,
    WDAElementNotDisappearError,
    WCAutoError,
    AppState,
)
